package com.uoc.christian.bttroute.CambioContrasena;


import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.christian.myapplication.backend.usuarioApi.UsuarioApi;
import com.example.christian.myapplication.backend.usuarioApi.model.Usuario;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import dataUtils.SharedPrefsUtils;
import com.uoc.christian.bttroute.R;

import java.io.IOException;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;
import dataUtils.ConexionBackend;

/**
 * A simple {@link Fragment} subclass.
 */
public class CambioContrasenaFragment extends Fragment implements CambioContrasenaView {


    @BindView(R.id.editEntradaPasswordActual)
    EditText editEntradaPasswordActual;
    @BindView(R.id.editEntradaPasswordNueva)
    EditText editEntradaPasswordNueva;
    @BindView(R.id.editEntradaPasswordRepite)
    EditText editEntradaPasswordRepite;
    @BindView(R.id.btnCambiarContrasena)
    Button btnCambiarContrasena;
    @BindView(R.id.progressBar)
    ProgressBar progressBar;
    Unbinder unbinder;

    CambioContrasenaPresenter presenter;
    //Servicio
    UsuarioApi servicio;
    String contrasenaActual ="";


    public CambioContrasenaFragment() {
        // Required empty public constructor
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        setupPresenter();
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_cambio_contrasena, container, false);
        unbinder = ButterKnife.bind(this, view);
        return view;

    }

    /*
     * Acciones que se realizan cuando se muestra o oculta el fragment
     */
    @Override
    public void onHiddenChanged(boolean hidden) {
        super.onHiddenChanged(hidden);
        if (!hidden) { // Se esta mostrando el fragment
            presenter.onCreate(this, getActivity().getApplicationContext());
        } else { // Se está ocultando el fragment
            presenter.onDestroy();
        }
    }

    /*
     * Cambiamos la contraseña.
     */
    @OnClick(R.id.btnCambiarContrasena)
    @Override
    public void handleCambiar() {
        if (presenter.comprobarCampos(editEntradaPasswordActual.getText().toString(), editEntradaPasswordNueva.getText().toString(), editEntradaPasswordRepite.getText().toString())) {
            new cambiarContrasena().execute(editEntradaPasswordNueva.getText().toString());
        }
    }

    /*
     * Mostramos un mensaje
     */
    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getActivity().getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Inicializamos el presentador
     */
    @Override
    public void setupPresenter() {
        presenter = new CambioContrasenaPresenterImplement(this, getActivity().getApplicationContext());
    }

    /*
     * Bloqueamos todos los campos
     */
    @Override
    public void bloquearCampos() {
        editEntradaPasswordActual.setEnabled(false);
        editEntradaPasswordNueva.setEnabled(false);
        editEntradaPasswordRepite.setEnabled(false);
        btnCambiarContrasena.setEnabled(false);
    }

    /*
     * Desbloqueamos los campos
     */
    @Override
    public void desbloquearBampos() {
        editEntradaPasswordActual.setEnabled(true);
        editEntradaPasswordNueva.setEnabled(true);
        editEntradaPasswordRepite.setEnabled(true);
        btnCambiarContrasena.setEnabled(true);
    }

    /*
     * Mostramos la barra de progreso según el parámetro recibido
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if (mostrar) {
            progressBar.setVisibility(View.VISIBLE);
        } else {
            progressBar.setVisibility(View.INVISIBLE);
        }
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        unbinder.unbind();
    }

    /*
     * Cambiamos la contraseña actual
     */
    protected class cambiarContrasena extends AsyncTask<String, Void, Usuario>{

        boolean contrasenaActualCoincide = true;

        @Override
        protected void onPreExecute(){
            contrasenaActual = editEntradaPasswordActual.getText().toString();
            mostrarProgressbar(true);
            bloquearCampos();
        }

        @Override
        protected Usuario doInBackground(String... value) {
            Usuario usuario = null;

            if(servicio == null){
                UsuarioApi.Builder builder = new UsuarioApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicio = builder.build();
            }

            try {
                //Comprobamos si el usuario introducido existe
                usuario = servicio.checkExist(SharedPrefsUtils.getStringPreference(getContext(), getString(R.string.key_usuario))).execute();
                if(usuario!=null){
                    if(presenter.comprobarContrasenaActual(usuario.getPassword(), contrasenaActual)) {
                        //Si existe, actualizamos la clave del usuario en la base de datos
                        servicio.updatePassword(usuario.getUser(), value[0]).execute();
                    }else{
                        contrasenaActualCoincide = false;
                    }
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            return usuario;
        }

        @Override
        protected void onPostExecute(Usuario usuario){
            mostrarProgressbar(false);
            desbloquearBampos();

            if(usuario != null) {
                if(!contrasenaActualCoincide){
                    mostrarToast(getString(R.string.cambioContrasena_mensaje_contrasenaActualNoIgual));
                }else {
                    mostrarToast(getString(R.string.cambioContrasena_mensaje_claveActualizada));
                    editEntradaPasswordNueva.setText("");
                    editEntradaPasswordActual.setText("");
                    editEntradaPasswordRepite.setText("");
                    editEntradaPasswordActual.requestFocus();
                }
            }
        }
    }
}
