package com.uoc.christian.bttroute.DetalleRutaServidor.Adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.example.christian.myapplication.backend.dificultadFisicaApi.model.DificultadFisica;
import com.example.christian.myapplication.backend.dificultadTecnicaApi.model.DificultadTecnica;
import com.example.christian.myapplication.backend.rutaApi.model.Ruta;
import com.example.christian.myapplication.backend.tipoRutaApi.model.TipoRuta;
import dataUtils.DatosFiltros;
import dataUtils.DatosRuta;
import com.uoc.christian.bttroute.R;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by christian on 9/11/17.
 */

public class RutasServidorAdapter extends RecyclerView.Adapter<RutasServidorAdapter.ViewHolder> {

    List<Ruta> listaRutas;
    OnItemClickListener onItemClickListener;
    LayoutInflater layoutInflater;
    Context context;


    public RutasServidorAdapter(List<Ruta> listaRutas, OnItemClickListener onItemClickListener, Context context) {
        this.listaRutas = listaRutas;
        this.onItemClickListener = onItemClickListener;
        this.context = context;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext())
                .inflate(R.layout.item_lista_ruta_servidor, parent, false);

        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        Ruta ruta = listaRutas.get(position);
        holder.textTituloRuta.setText(ruta.getTituloRuta());
        holder.textDatoDesnivel.setText(ruta.getDesnivelPositivo() + " m");

        for(DificultadFisica df: DatosFiltros.listaDificultadFisica){
            if(df.getId().toString().equals(ruta.getIdDificultadFisica().toString())){
                holder.textDatoDifFisica.setText(df.getDescripcion());
            }
        }

        for(DificultadTecnica dt: DatosFiltros.listaDificultadTecnica){
            if(dt.getId().toString().equals(ruta.getIdDificultadTecnica().toString())){
                holder.textDatoDifTecnica.setText(dt.getDescripcion());
            }
        }

        for(TipoRuta tr: DatosFiltros.listaTipoRuta){
            if(tr.getId().toString().equals(ruta.getIdTipoRuta().toString())){
                holder.textDatoTipoRuta.setText(tr.getDescripcion());
            }
        }

        holder.textDatoDistancia.setText(ruta.getDistancia().toString() + " Km");
        holder.setOnClickListener(ruta, onItemClickListener);
    }

    @Override
    public int getItemCount() {
        return listaRutas.size();
    }

    public void setRutas(List<Ruta> lista){
        this.listaRutas.addAll(lista);
        notifyDataSetChanged();
    }


    protected class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.textTituloRuta)
        TextView textTituloRuta;
        @BindView(R.id.textDatoDistancia)
        TextView textDatoDistancia;
        @BindView(R.id.textDatoDesnivel)
        TextView textDatoDesnivel;
        @BindView(R.id.textDatoDifTecnica)
        TextView textDatoDifTecnica;
        @BindView(R.id.textDatoDifFisica)
        TextView textDatoDifFisica;
        @BindView(R.id.textDatoTipoRuta4)
        TextView textDatoTipoRuta;

        View view;

        public ViewHolder(View itemView) {
            super(itemView);
            this.view = itemView;
            ButterKnife.bind(this, view);
        }

        private void setOnClickListener(final Ruta ruta, final OnItemClickListener listener){
            view.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    DatosRuta datosRuta = new DatosRuta();
                    datosRuta.setTituloRuta(ruta.getTituloRuta());
                    datosRuta.setObservaciones(ruta.getObservaciones());
                    datosRuta.setIdDificultadFisica(ruta.getIdDificultadFisica());
                    datosRuta.setIdDificultadTecnica(ruta.getIdDificultadTecnica());
                    datosRuta.setTiempo(ruta.getTiempo());
                    datosRuta.setDistancia(ruta.getDistancia());
                    datosRuta.setDesnivelPositivo(ruta.getDesnivelPositivo());
                    datosRuta.setVelocidadMaxima(ruta.getVelocidadMaxima());
                    datosRuta.setVelocidadMedia(ruta.getVelocidadMedia());
                    datosRuta.setIdTipoRuta(ruta.getIdTipoRuta());
                    datosRuta.setCompartida(ruta.getCompartida());
                    datosRuta.setIdRuta(ruta.getId());
                    datosRuta.setGeoPointsCodificados(ruta.getGeoPointsCodificados());
                    datosRuta.setDesnivelNegativo(ruta.getDesnivelNegativo());
                    datosRuta.setDesnivelAcumulado(ruta.getDesnivelAcumulado());

                    listener.onItemClick(datosRuta);
                }
            });
        }
    }
}
