package com.uoc.christian.bttroute.ExploradorArchivos;

import android.os.Bundle;
import android.os.Environment;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;

import dataUtils.Constants;

import com.uoc.christian.bttroute.PantallaInicial.PantallaInicialActivity;
import com.uoc.christian.bttroute.R;

import org.osmdroid.util.GeoPoint;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.Unbinder;

/*
 * http://programandoointentandolo.com/2014/06/como-hacer-un-explorador-de-archivos-en-android.html
 */
public class ExploradorArchivosFragment extends Fragment implements ExploradorArchivosView {

    @BindView(android.R.id.list)
    ListView list;
    @BindView(R.id.textCarpetaVacia)
    TextView textCarpetaVacia;

    Unbinder unbinder;

    private File directorioActual;
    private List<String> listaNombreArchivos;
    private List<String> listaRutasArchivos;

    ExploradorArchivosPresenter presenter;


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View view = inflater.inflate(R.layout.fragment_explorador_archivos, container, false);
        //setContentView(R.layout.fragment_explorador_archivos);
        unbinder = ButterKnife.bind(this, view);

        setupPresneter();

        //Obtenemos el directorio actual
        directorioActual = new File(Environment.getExternalStorageDirectory(), Constants.carpetaArchivos);

        //Vemos los archivos
        verArchivosDirectorio(directorioActual);

        return view;
    }

    @Override
    public void verArchivosDirectorio(File directorioActual) {
        listaNombreArchivos = new ArrayList<>();
        final File[] listaArchivos = directorioActual.listFiles();
        listaRutasArchivos = new ArrayList<>();

        for (File archivo : listaArchivos) {
            listaRutasArchivos.add(archivo.getPath());
        }

        for (int i = 0; i < listaRutasArchivos.size(); i++) {
            File archivo = new File(listaRutasArchivos.get(i));
            //Solo mostramos los archivos con extensión GPX
            if (archivo.isFile() && archivo.getName().endsWith(".gpx")) {
                listaNombreArchivos.add(archivo.getName().toString());
            }
        }

        //Según si hay archivos o no, mostramos los archivos o un comentario.
        if (listaArchivos.length < 1) {
            textCarpetaVacia.setVisibility(View.VISIBLE);
            list.setVisibility(View.GONE);
        }else{
            textCarpetaVacia.setVisibility(View.GONE);
            list.setVisibility(View.VISIBLE);
        }

        list.setAdapter(new ArrayAdapter<String>(getContext(), R.layout.item_lista_archivos, R.id.Itemname, listaNombreArchivos));

        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> adapterView, View view, int i, long l) {
                File archivo = new File(listaNombreArchivos.get(i));

                List<GeoPoint> lista = presenter.getPointsGPX(archivo);

                if (lista.size() > 0) {

                    String idGPXFile = "";

                    //Comprobamos si tiene el campo id de ruta en el archivo. Se hace esta comprobación porqué podemos cargar
                    //archivos GPX procedentes de otras fuentes.
                    if (presenter.getLinkRutaGPXFile(archivo) != null) {
                        idGPXFile = presenter.getLinkRutaGPXFile(archivo);
                    }

                    //Llamámos al método de la pantalla inicial que nos llevará a la pantalla de crear ruta pero mostrando el recorrido de la ruta
                    ((PantallaInicialActivity) getActivity()).cargarRuta(false, idGPXFile, lista);
                }

            }
        });

    }

    /*
     * Seteamos el presentador
     */
    @Override
    public void setupPresneter() {
        presenter = new ExploradorArchivosPresenterImplement(this, new ArrayList<GeoPoint>());
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        unbinder.unbind();
    }
}
