package com.uoc.christian.bttroute.ExploradorArchivos;

import android.location.Location;
import android.os.Environment;


import org.osmdroid.util.GeoPoint;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

/**
 * Created by christian on 26/10/17.
 */

public class ExploradorArchivosPresenterImplement implements ExploradorArchivosPresenter {

    ExploradorArchivosView exploradorArchivosView;
    List<GeoPoint> pointsGPX;

    public ExploradorArchivosPresenterImplement(ExploradorArchivosView exploradorArchivosView, List<GeoPoint> geoPoints){
        this.exploradorArchivosView = exploradorArchivosView;
        pointsGPX = geoPoints;
        onCreate(this.exploradorArchivosView, this.pointsGPX);
    }

    @Override
    public void onCreate(ExploradorArchivosView exploradorArchivosView, List<GeoPoint> geoPoints) {
        this.exploradorArchivosView = exploradorArchivosView;
        this.pointsGPX = geoPoints;
    }

    @Override
    public void onDestroy() {
        exploradorArchivosView = null;
        pointsGPX = null;
    }

    @Override
    public List<GeoPoint> getPointsGPX(File gpxFile) {
        List<Location> locations = decodeGPX(gpxFile);

        if(pointsGPX != null && pointsGPX.size()>0) {
            pointsGPX.clear();
        }

        if(locations!=null && locations.size()>0) {
            for (int i = 0; i < locations.size(); i++) {
                Location location = locations.get(i);
                GeoPoint geoPoint = new GeoPoint(location.getLatitude(), location.getLongitude());
                pointsGPX.add(geoPoint);
            }
        }

        return pointsGPX;
    }

    @Override
    public List<Location> decodeGPX(File gpxFile) {

        List<Location> list = new ArrayList<>();

        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();

        try {
            DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
            FileInputStream fileInputStream = new FileInputStream(Environment.getExternalStorageDirectory() + "/BttRouteMaps/"+gpxFile);
            Document document = documentBuilder.parse(fileInputStream);
            Element element = document.getDocumentElement();

            NodeList nodeList_trkpt = element.getElementsByTagName("trkpt");

            for(int i=0; i<nodeList_trkpt.getLength(); i++){
                Node node = nodeList_trkpt.item(i);
                NamedNodeMap attributes = node.getAttributes();

                String latitude = attributes.getNamedItem("lat").getTextContent();
                Double latitude_double = Double.parseDouble(latitude);
                String longitude = attributes.getNamedItem("lon").getTextContent();
                Double longitud_double = Double.parseDouble(longitude);

                String locationName = "Location " + i;
                Location location = new Location(locationName);
                location.setLatitude(latitude_double);
                location.setLongitude(longitud_double);

                list.add(location);
            }
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return list;
    }

    @Override
    public String getLinkRutaGPXFile(File gpxFile) {
        String valor = null;
        DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder documentBuilder = null;
        try {
            documentBuilder = documentBuilderFactory.newDocumentBuilder();
            FileInputStream fileInputStream = new FileInputStream(Environment.getExternalStorageDirectory() + "/BttRouteMaps/"+gpxFile);
            Document document = documentBuilder.parse(fileInputStream);
            Element element = document.getDocumentElement();

            NodeList node_linkRuta = element.getElementsByTagName("linkRuta");

            if(node_linkRuta!=null && node_linkRuta.getLength()>0){
                Node node = node_linkRuta.item(0);
                valor = node.getTextContent();
            }
        } catch (ParserConfigurationException e) {
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (SAXException e) {
            e.printStackTrace();
        }

        return valor;
    }
}
