package com.uoc.christian.bttroute.GuardarRuta;

import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.crashlytics.android.Crashlytics;
import com.example.christian.myapplication.backend.dificultadFisicaApi.model.DificultadFisica;
import com.example.christian.myapplication.backend.dificultadTecnicaApi.model.DificultadTecnica;
import com.example.christian.myapplication.backend.rutaApi.RutaApi;
import com.example.christian.myapplication.backend.rutaApi.model.Ruta;
import com.example.christian.myapplication.backend.tipoRutaApi.model.TipoRuta;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import com.uoc.christian.bttroute.GuardarRuta.Adapter.AdapterSpinnerDifFisica;
import com.uoc.christian.bttroute.GuardarRuta.Adapter.AdapterSpinnerDifTecnica;
import com.uoc.christian.bttroute.GuardarRuta.Adapter.AdapterSpinnerTipoRuta;
import com.uoc.christian.bttroute.PantallaInicial.PantallaInicialActivity;
import com.uoc.christian.bttroute.R;

import java.io.IOException;
import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.ConexionBackend;
import dataUtils.Constants;
import dataUtils.DatosRuta;
import dataUtils.ListasRutas;
import dataUtils.SharedPrefsUtils;
import io.fabric.sdk.android.Fabric;

public class GuardarRutaActivity extends AppCompatActivity implements GuardarRutaView {

    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.spinnerDifTecnica)
    Spinner spinnerDifTecnica;
    @BindView(R.id.spinnerDifFisica)
    Spinner spinnerDifFisica;
    @BindView(R.id.spinnerTipoRuta)
    Spinner spinnerTipoRuta;
    @BindView(R.id.editTextObservaciones)
    EditText editTextObservaciones;
    @BindView(R.id.progressbar)
    ProgressBar progressbar;
    @BindView(R.id.editTextTituloRuta)
    EditText editTextTituloRuta;
    @BindView(R.id.btnFinalizar)
    Button btnFinalizar;
    @BindView(R.id.textObservaciones)
    TextView textObservaciones;

    private DatosRuta datosRuta = null;
    private RutaApi servicioRuta = null;
    private Ruta ruta = null;
    private GuardarRutaPresenterImplement presenter = null;

    private AdapterSpinnerDifFisica adapaterSpinnerDifFisica;
    private AdapterSpinnerDifTecnica adapterSpinnerDifTecnica;
    private AdapterSpinnerTipoRuta adapterSpinnerTipoRuta;

    private boolean tenemosConexion = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_guardar_ruta);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        setupActionbar();
        setupPresenter();
        inflateSpinnerDifFisica();
        inflateSpinnerDifTecnica();
        inflateSpinnerTipoRuta();

        Bundle bundle = getIntent().getExtras();
        datosRuta = (DatosRuta) bundle.getSerializable(Constants.datosRutaUsuario);
    }

    /*
     * Seteamos el presentador
     */
    @Override
    public void setupPresenter() {
        presenter = new GuardarRutaPresenterImplement(this, getApplicationContext());
    }

    /*
     * Seteamos la actionbar
     */
    @Override
    public void setupActionbar() {
        appbar.setTitle(getString(R.string.app_name));
        appbar.setSubtitle(getString(R.string.guardar_ruta_titulo));
        setSupportActionBar(appbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    /*
     * Acción realizada al pulsar el botón Finalizar.
     */
    @OnClick(R.id.btnFinalizar)
    @Override
    public void handleFinalizar() {
        //Comprobamos que el usuario ha introducido un título
        if (presenter.comprobarCampos(editTextTituloRuta.getText().toString())) {
            //Creamos un nuevo objeto Ruta
            ruta = new Ruta();
            ruta.setId(presenter.obtenerUUID());
            ruta.setTituloRuta(editTextTituloRuta.getText().toString().replace("'", ""));
            ruta.setDesnivelPositivo(datosRuta.getDesnivelPositivo());
            ruta.setTiempo(datosRuta.getTiempo());
            ruta.setDistancia(datosRuta.getDistancia());
            ruta.setVelocidadMaxima(datosRuta.getVelocidadMaxima());
            ruta.setVelocidadMedia(datosRuta.getVelocidadMedia());

            DificultadFisica dificultadFisica = (DificultadFisica) spinnerDifFisica.getSelectedItem();
            ruta.setIdDificultadFisica(Integer.valueOf(dificultadFisica.getId().toString()));

            DificultadTecnica dificultadTecnica = (DificultadTecnica) spinnerDifTecnica.getSelectedItem();
            ruta.setIdDificultadTecnica(Integer.valueOf(dificultadTecnica.getId().toString()));

            TipoRuta tipoRuta = (TipoRuta) spinnerTipoRuta.getSelectedItem();
            ruta.setIdTipoRuta(Integer.valueOf(tipoRuta.getId().toString()));

            ruta.setCompartida(false);

            //Comprobamos de que forma se ha realizado el login en la aplicación. Si se ha hecho con una cuenta de facebook o con una cuenta registrada
            if (!SharedPrefsUtils.getBooleanPreference(getApplicationContext(), getString(R.string.key_isLoginFacebook), false)) {
                ruta.setIdUsuario(SharedPrefsUtils.getLongPreference(getApplicationContext(), getString(R.string.key_idusuario), 0));
                ruta.setUsuarioFacebook(null);
            } else {
                ruta.setIdUsuario(null);
                ruta.setUsuarioFacebook(SharedPrefsUtils.getStringPreference(getApplicationContext(), getString(R.string.key_idUsuarioFacebook)));
            }

            ruta.setObservaciones(editTextObservaciones.getText().toString().replace("'", "´"));

            ruta.setGeoPointsCodificados(datosRuta.getGeoPointsCodificados());
            ruta.setDesnivelAcumulado(datosRuta.getDesnivelAcumulado());
            ruta.setDesnivelNegativo(datosRuta.getDesnivelNegativo());

            //Comprobamos si la ruta procede de un archivo descargado de nuestro servidor. Los GPX que se descargan, tienen el ID con el que se
            //han registrado. Este ID se guarda en un sharedpreferences y aquí lo comprobamos.
            if (SharedPrefsUtils.getStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen)) != null) {
                ruta.setIdRutaOriginal(SharedPrefsUtils.getStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen)));
            } else {
                ruta.setIdRutaOriginal(null);
            }
            //Ejecutamos el proceso de guardado
            new guardarRuta().execute(ruta);
            //Añadimos la ruta a la lista
            if(ListasRutas.listaMisRutas!=null && ListasRutas.listaMisRutas.size()>0) {
                ListasRutas.listaMisRutas.add(ruta);
            }else{
                ListasRutas.listaMisRutas = new ArrayList<>();
                ListasRutas.listaMisRutas.add(ruta);
            }
        }
    }

    /*
     * Cargamos los datos del spinner dificultad física
     */
    @Override
    public void inflateSpinnerDifFisica() {
        adapaterSpinnerDifFisica = new AdapterSpinnerDifFisica(this, android.R.layout.simple_spinner_item, presenter.getListaDifFisica());
        adapaterSpinnerDifFisica.setDropDownViewResource(R.layout.item_spinner);
        spinnerDifFisica.setAdapter(adapaterSpinnerDifFisica);
    }

    /*
     * Cargamos los datos del spinner dificultad técnica
     */
    @Override
    public void inflateSpinnerDifTecnica() {
        adapterSpinnerDifTecnica = new AdapterSpinnerDifTecnica(this, android.R.layout.simple_spinner_item, presenter.getListaDifTecnica());
        adapterSpinnerDifTecnica.setDropDownViewResource(R.layout.item_spinner);
        spinnerDifTecnica.setAdapter(adapterSpinnerDifTecnica);
    }

    /*
     * Cargamos los datos del spinner tipo de ruta
     */
    @Override
    public void inflateSpinnerTipoRuta() {
        adapterSpinnerTipoRuta = new AdapterSpinnerTipoRuta(this, android.R.layout.simple_spinner_item, presenter.getListaTipoRuta());
        adapterSpinnerTipoRuta.setDropDownViewResource(R.layout.item_spinner);
        spinnerTipoRuta.setAdapter(adapterSpinnerTipoRuta);
    }

    /*
     * Bloqueamos los campos
     */
    @Override
    public void bloquearCampos() {
        spinnerDifTecnica.setEnabled(false);
        spinnerDifFisica.setEnabled(false);
        spinnerTipoRuta.setEnabled(false);
        editTextObservaciones.setEnabled(false);
        editTextTituloRuta.setEnabled(false);
        btnFinalizar.setEnabled(false);
    }

    /*
     * Desbloqueamos los campos
     */
    @Override
    public void desbloquearBampos() {
        spinnerDifTecnica.setEnabled(true);
        spinnerDifFisica.setEnabled(true);
        spinnerTipoRuta.setEnabled(true);
        editTextObservaciones.setEnabled(true);
        editTextTituloRuta.setEnabled(true);
        btnFinalizar.setEnabled(true);
    }

    /*
     * Mostramos o ocultamos el progressbar según el valor recibido
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if (mostrar) {
            progressbar.setVisibility(View.VISIBLE);
        } else {
            progressbar.setVisibility(View.INVISIBLE);
        }
    }

    /*
     * Mostramos un toast en pantalla
     */
    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Método asincrono para guardar la ruta en el servidor
     */
    protected class guardarRuta extends AsyncTask<Ruta, Void, Ruta> {

        @Override
        protected void onPreExecute() {
            mostrarProgressbar(true);
            bloquearCampos();
        }

        @Override
        protected Ruta doInBackground(Ruta... datosRutas) {

            //Primero comprobamos que tengamos conexión a internet
            if (presenter.comprobarConexion()) {

                tenemosConexion = true;

                if (servicioRuta == null) {
                    RutaApi.Builder build = new RutaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                            .setRootUrl(ConexionBackend.url);

                    servicioRuta = build.build();
                }

                try {
                    return servicioRuta.insert(datosRutas[0]).execute();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                tenemosConexion = false;
            }

            return null;
        }

        @Override
        protected void onPostExecute(Ruta ruta) {
            mostrarProgressbar(false);
            desbloquearBampos();
            if (ruta != null) {
                //Reseteamos el sharedpreference que guarda el id de ruta de origen
                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen), null);
                //Volvemos a la pantalla inicial.
                Intent intent = new Intent(GuardarRutaActivity.this, PantallaInicialActivity.class);
                startActivity(intent);
                finish();
            } else {
                if (tenemosConexion) {
                    mostrarToast(getString(R.string.guardarRuta_mensaje_falloAlGuardar));
                } else {
                    mostrarToast(getString(R.string.sinConexion));
                }
            }
        }
    }
}
