package com.uoc.christian.bttroute.Login;

import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.Signature;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.util.Base64;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import com.example.christian.myapplication.backend.dificultadFisicaApi.DificultadFisicaApi;
import com.example.christian.myapplication.backend.dificultadTecnicaApi.DificultadTecnicaApi;
import com.example.christian.myapplication.backend.tipoRutaApi.TipoRutaApi;
import com.example.christian.myapplication.backend.usuarioApi.model.Usuario;
import com.example.christian.myapplication.backend.usuarioApi.UsuarioApi;
import com.facebook.AccessToken;
import com.facebook.CallbackManager;
import com.facebook.FacebookCallback;
import com.facebook.FacebookException;
import com.facebook.FacebookSdk;
import com.facebook.Profile;
import com.facebook.ProfileTracker;
import com.facebook.appevents.AppEventsLogger;
import com.facebook.login.LoginManager;
import com.facebook.login.LoginResult;
import com.facebook.login.widget.LoginButton;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import dataUtils.Constants;
import dataUtils.DatosFiltros;
import dataUtils.SharedPrefsUtils;
import com.uoc.christian.bttroute.PantallaInicial.PantallaInicialActivity;
import com.uoc.christian.bttroute.R;
import com.uoc.christian.bttroute.RecordarPassword.RecordarPasswordActivity;
import com.uoc.christian.bttroute.RegistroUsuario.RegistroActivity;

import com.crashlytics.android.Crashlytics;

import io.fabric.sdk.android.Fabric;

import java.io.IOException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.ConexionBackend;


public class LoginActivity extends AppCompatActivity implements LoginView {

    @BindView(R.id.editEntradaUsuario)
    EditText editEntradaUsuario;
    @BindView(R.id.editEntradaPassword)
    EditText editEntradaPassword;
    @BindView(R.id.btnAceptar)
    Button btnAceptar;
    @BindView(R.id.btnFacebook)
    LoginButton btnFacebook;
    @BindView(R.id.txtRegistrarse)
    TextView txtRegistrarse;
    @BindView(R.id.txtOlvidaste)
    TextView txtOlvidaste;
    @BindView(R.id.progressBar)
    ProgressBar progressBar;

    //Resultado para la operacion onActivityResult
    private final static int REQUEST_REGISTRO=200;
    //Presentador
    LoginPresenterImplement loginPresenterImplement;
    //Servicio
    private UsuarioApi servicio = null;
    private DificultadFisicaApi servicioDifFisica = null;
    private DificultadTecnicaApi servicioDifTecnica = null;
    private TipoRutaApi servicioTipoRuta = null;
    CallbackManager callbackManager;
    //Variable utilizada en el onBackpressed
    private long backPressedTime = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_login);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        comprobarPermisos();
        setupPresenter();

        //Generamos una hash key para el login de Facebook. Este hash lo tenemos que introducir en la
        //consola de desarrollo de facebook en el caso de que estemos debugando la aplicación.
        try {
            PackageInfo info = getPackageManager().getPackageInfo(
                    "com.uoc.christian.bttroute",
                    PackageManager.GET_SIGNATURES);
            for (Signature signature : info.signatures) {
                MessageDigest md = MessageDigest.getInstance("SHA");
                md.update(signature.toByteArray());
                Log.d("KeyHash:", Base64.encodeToString(md.digest(), Base64.DEFAULT));
            }
        } catch (PackageManager.NameNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        //Al iniciar la aplicación, eliminamos todos los archivos de preferencias
        SharedPrefsUtils.clearSharedPreferences(getApplicationContext());
        //Comprobamos si tenemos una sesión de facebook iniciada y en caso afirmativo la cerramos
        if(AccessToken.getCurrentAccessToken()!=null){
            LoginManager.getInstance().logOut();
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data){
        if(REQUEST_REGISTRO == requestCode){
            if(resultCode == RESULT_OK) {
                Bundle bundle = data.getExtras();
                String usuario = bundle.getString(Constants.usuario);
                String password = bundle.getString(Constants.password);
                String[] values = {usuario, password};
                new comprobarUsuarioLogin().execute(values);
            }
        }else {
            callbackManager.onActivityResult(requestCode,resultCode, data);
        }
    }

    /*
     * Inicializamos el presentador
     */
    @Override
    public void setupPresenter() {
        loginPresenterImplement = new LoginPresenterImplement(this,getApplicationContext());
    }

    /*
     * Validamos los datos del usuario
     */
    @OnClick(R.id.btnAceptar)
    @Override
    public void handleValidar() {
        if(loginPresenterImplement.comprobarConexion()) {
            //Primero comprobamos que el usuario no se deje ningún campo vacío.
            if (loginPresenterImplement.comprobarDatos(editEntradaUsuario.getText().toString(), editEntradaPassword.getText().toString())) {
                String[] values = {editEntradaUsuario.getText().toString(), editEntradaPassword.getText().toString()};
                new comprobarUsuarioLogin().execute(values);
            }
        }else{
            mostrarToastError(getResources().getString(R.string.sinConexion));
        }
    }

    /*
     * Accedemos al formulario de registro
     */
    @OnClick(R.id.txtRegistrarse)
    @Override
    public void handleRegistrar() {
        Intent intent = new Intent(LoginActivity.this, RegistroActivity.class);
        startActivityForResult(intent, REQUEST_REGISTRO);
    }

    /*
     * Accedemos al formulario para recordar el password
     */
    @OnClick(R.id.txtOlvidaste)
    @Override
    public void handleRecordarPassword() {
        Intent intent = new Intent(LoginActivity.this, RecordarPasswordActivity.class);
        startActivity(intent);
    }

    /*
     * Realizamos login a través de la cuenta de facebook
     */
    @OnClick(R.id.btnFacebook)
    //@Override
    public void handleLoginFacebook() {
        if(loginPresenterImplement.comprobarConexion()) {
            //bloquearCampos();
            btnFacebook.setReadPermissions("email");
            callbackManager = CallbackManager.Factory.create();
            btnFacebook.registerCallback(callbackManager, new FacebookCallback<LoginResult>() {
                @Override
                public void onSuccess(LoginResult loginResult) { //Si el resultado ha tenido éxito, accedemos a la pantalla principal
                    ProfileTracker profileTracker;
                    Profile profile = Profile.getCurrentProfile();

                    if (profile == null) {
                        profileTracker = new ProfileTracker() {
                            @Override
                            protected void onCurrentProfileChanged(Profile oldProfile, Profile currentProfile) {
                                this.stopTracking();
                                Profile.setCurrentProfile(currentProfile);
                                desbloquearCampos();
                                SharedPrefsUtils.setBooleanPreference(getApplicationContext(), getString(R.string.key_isLoginFacebook), true);
                                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_usuario), Profile.getCurrentProfile().getName());
                                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idUsuarioFacebook), Profile.getCurrentProfile().getId().toString());

                                //Recuperamos los filtros del backend y que utilizaremos más adelante. Esto nos permite
                                //tenerlos cargados desde el inicio de la app.
                                new obtenerFiltros().execute();

                                Intent intent = new Intent(LoginActivity.this, PantallaInicialActivity.class);
                                startActivity(intent);
                                finish();
                            }
                        };
                        profileTracker.startTracking();
                    } else {
                        desbloquearCampos();
                        SharedPrefsUtils.setBooleanPreference(getApplicationContext(), getString(R.string.key_isLoginFacebook), true);
                        SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_usuario), Profile.getCurrentProfile().getName());
                        SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idUsuarioFacebook), Profile.getCurrentProfile().getId().toString());

                        //Recuperamos los filtros del backend y que utilizaremos más adelante. Esto nos permite
                        //tenerlos cargados desde el inicio de la app.
                        new obtenerFiltros().execute();

                        Intent intent = new Intent(LoginActivity.this, PantallaInicialActivity.class);
                        startActivity(intent);
                        finish();
                    }
                }

                @Override
                public void onCancel() {
                    //nothing
                }

                @Override
                public void onError(FacebookException error) {
                    desbloquearCampos();
                    mostrarToastError(error.toString());
                }
            });
        }else{
            mostrarToastError(getResources().getString(R.string.sinConexion));
        }
    }

    /*
     * Mostramos los mensajes de error
     */
    @Override
    public void mostrarToastError(String mensaje) {
        Toast.makeText(getApplicationContext(),mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Mostramos o ocultamos el progressbar
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if(mostrar) {
            progressBar.setVisibility(View.VISIBLE);
        }else{
            progressBar.setVisibility(View.INVISIBLE);
        }
    }

    /*
     * Bloqueamos los campos
     */
    @Override
    public void bloquearCampos() {
        editEntradaPassword.setEnabled(false);
        editEntradaUsuario.setEnabled(false);
        btnAceptar.setEnabled(false);
        btnFacebook.setEnabled(false);
        txtOlvidaste.setEnabled(false);
        txtRegistrarse.setEnabled(false);
    }

    /*
     * Desbloqueamos los campos
     */
    @Override
    public void desbloquearCampos() {
        editEntradaPassword.setEnabled(true);
        editEntradaUsuario.setEnabled(true);
        btnAceptar.setEnabled(true);
        btnFacebook.setEnabled(true);
        txtOlvidaste.setEnabled(true);
        txtRegistrarse.setEnabled(true);
    }

    @Override
    public void onBackPressed() {
        long t = System.currentTimeMillis();
        if (t - backPressedTime > 2000) {    // 2 secs
            backPressedTime = t;
            Toast.makeText(this, "Presiona 'volver' para salir de la aplicación",
                    Toast.LENGTH_SHORT).show();
        } else {
            // clean up
            //super.onBackPressed();
            finish();
            //System.exit(0);
        }
    }

    /*
     * Realizamos la comprobación del usuario con el backend
     */
    private class comprobarUsuarioLogin extends AsyncTask<String[], Void, Usuario>{

        @Override
        protected void onPreExecute(){
            mostrarProgressbar(true);
            bloquearCampos();
        }

        @Override
        protected Usuario doInBackground(String[]... params) {
            if (servicio == null) {
                UsuarioApi.Builder builder = new UsuarioApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicio = builder.build();
            }

            try {
                String u = null;
                String p = null;

                for (String[] s : params) {
                    u = s[0];
                    p = s[1];
                }

                //Devolvemos el usuario
                return servicio.get(u, p).execute();
            } catch (IOException e) {
                e.printStackTrace();
            }

            return null;
        }

        @Override
        protected void onPostExecute(Usuario usuario){
            desbloquearCampos();

            if(progressBar.getVisibility()==View.VISIBLE){
                mostrarProgressbar(false);
            }

            if (usuario != null) {
                //Guardamos los datos en memória
                SharedPrefsUtils.setLongPreference(getApplicationContext(), getString(R.string.key_idusuario), usuario.getId());
                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_usuario), usuario.getUser());
                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_password), usuario.getPassword());
                SharedPrefsUtils.setBooleanPreference(getApplicationContext(), getString(R.string.key_isLoginFacebook), false);


                //Recuperamos los filtros del backend y que utilizaremos más adelante. Esto nos permite
                //tenerlos cargados desde el inicio de la app.
                new obtenerFiltros().execute();

                Intent intent = new Intent(LoginActivity.this, PantallaInicialActivity.class);
                startActivity(intent);
                finish();

            } else {
                mostrarToastError(getResources().getString(R.string.login_failed));
            }
        }
    }

    /*
     * Recuperamos inicialmente todos los datos que utilizamos en los spinners. Así nos evitamos
     * que cargarlos en el momento de cargar la pantalla de guardar ruta o pantalla de filtros.
     * Al hacerlo en background es totalmente transparente para el usuario
     */
    private class obtenerFiltros extends AsyncTask<Void, Void, Void>{
        @Override
        protected Void doInBackground(Void... voids) {
            if(servicioDifFisica == null){
                DificultadFisicaApi.Builder builder = new DificultadFisicaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicioDifFisica = builder.build();
            }

            if(servicioDifTecnica == null){
                DificultadTecnicaApi.Builder builder = new DificultadTecnicaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicioDifTecnica = builder.build();
            }

            if(servicioTipoRuta == null){
                TipoRutaApi.Builder builder = new TipoRutaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicioTipoRuta = builder.build();
            }

            try {
                DatosFiltros.listaDificultadFisica=servicioDifFisica.list().execute().getItems();
                DatosFiltros.listaDificultadTecnica=servicioDifTecnica.list().execute().getItems();
                DatosFiltros.listaTipoRuta=servicioTipoRuta.list().execute().getItems();
            }catch (IOException e){
                e.printStackTrace();
            }
            return null;
        }
    }

    ////////////////////// COMPROBAR PERMISOS DE ESCRITURA /////////////////////

    private void comprobarPermisos(){
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (!checkPermission()) {
                requestPermission();
            }
        }
    }

    protected boolean checkPermission() {
        int result = ContextCompat.checkSelfPermission(this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE);
        if (result == PackageManager.PERMISSION_GRANTED) {
            return true;
        } else {
            return false;
        }
    }

    protected void requestPermission() {

        if (ActivityCompat.shouldShowRequestPermissionRationale(this, android.Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
            Toast.makeText(this, "BttRoute necesita tener acceso a los archivos locales.\nAceptas?", Toast.LENGTH_LONG).show();
        } else {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                requestPermissions(new String[]{android.Manifest.permission.WRITE_EXTERNAL_STORAGE}, 100);
            }
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        switch (requestCode) {
            case 100:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    //do your work
                } else {
                    Log.e("value", "Permiso denegado.");
                }
                break;
        }
    }
}
