package com.uoc.christian.bttroute.MapaAmpliado;

import android.graphics.Color;
import android.location.Location;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.widget.ImageView;

import dataUtils.Constants;

import com.uoc.christian.bttroute.R;

import org.osmdroid.config.Configuration;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapController;
import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.Marker;
import org.osmdroid.views.overlay.Polyline;
import org.osmdroid.views.overlay.compass.CompassOverlay;
import org.osmdroid.views.overlay.compass.InternalCompassOrientationProvider;
import org.osmdroid.views.overlay.gestures.RotationGestureOverlay;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;

public class MapaAmpliadoActivity extends AppCompatActivity implements MapaAmpliadoView {


    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.mapaViewAmpliado)
    MapView mapaViewAmpliado;
    @BindView(R.id.imgCenter)
    ImageView imgCenter;

    private String geoPointsCodificados = "";
    private List<GeoPoint> listaGeoPoints;
    private MapaAmpliadoPresenterImplement presenter;
    private MapController mapController;
    private String tituloRuta = "";
    private RotationGestureOverlay rotationGestureOverlay;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_mapa_ampliado);
        ButterKnife.bind(this);
        Configuration.getInstance().load(getApplicationContext(), PreferenceManager.getDefaultSharedPreferences(getApplicationContext()));
        setupPresenter();
        getExtras();
        setupActionbar();

        //Ativamos el compás del mapa
        CompassOverlay mCompassOverlay = new CompassOverlay(getApplicationContext(),
                new InternalCompassOrientationProvider(getApplicationContext()),
                mapaViewAmpliado);
        mCompassOverlay.enableCompass();
        mapaViewAmpliado.getOverlays().add(mCompassOverlay);

        //mapaViewAmpliado.setTileSource(TileSourceFactory.HIKEBIKEMAP);

        if (!geoPointsCodificados.isEmpty())
            decodeGeopoints(geoPointsCodificados);


    }

    @Override
    protected void onResume(){
        super.onResume();
        Configuration.getInstance().load(this, PreferenceManager.getDefaultSharedPreferences(this));
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {

        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;

        }

        return super.onOptionsItemSelected(item);
    }

    /*
     * Seteamos la actionbar
     */
    @Override
    public void setupActionbar() {
        appbar.setTitle(getString(R.string.app_name));
        appbar.setSubtitle(tituloRuta);
        setSupportActionBar(appbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }

    /*
     * Setemaos el presentador
     */
    @Override
    public void setupPresenter() {
        presenter = new MapaAmpliadoPresenterImplement();
    }

    /*
     * Recibimos los parámetros del intent
     */
    @Override
    public void getExtras() {
        Bundle bundle = getIntent().getExtras();
        geoPointsCodificados = bundle.getString(Constants.geoPointsCodificados);
        tituloRuta = bundle.getString(Constants.tituloRuta);
    }

    /*
     * Recibimos una lista con los geopoints
     */
    @Override
    public void decodeGeopoints(String geoPointsCodificados) {
        listaGeoPoints = new ArrayList<>();
        listaGeoPoints = presenter.decodeGeopoints(geoPointsCodificados);
        marcarRecorrido();
    }

    /*
     * Mostramos el recorrido de la ruta
     */
    @Override
    public void marcarRecorrido() {

        mapController = (MapController) mapaViewAmpliado.getController();
        mapController.setZoom(15);
        mapaViewAmpliado.setMultiTouchControls(true);
        GeoPoint geoPoint = new GeoPoint(listaGeoPoints.get(0).getLatitude(), listaGeoPoints.get(0).getLongitude());
        mapController.setCenter(geoPoint);
        mapaViewAmpliado.setMaxZoomLevel(30);

        //Activamos la rotación del mapa
        rotationGestureOverlay = new RotationGestureOverlay(mapaViewAmpliado);
        rotationGestureOverlay.setEnabled(true);
        mapaViewAmpliado.getOverlays().add(rotationGestureOverlay);

        Polyline polyline = new Polyline();
        polyline.setGeodesic(true);
        polyline.setPoints(listaGeoPoints);
        polyline.setWidth(15f);
        polyline.setColor(Color.BLUE);

        mapaViewAmpliado.getOverlays().add(polyline);

        //Añadimos las marcas de inicio y meta en la ruta
        for (int i = 0; i < listaGeoPoints.size(); i++) {
            if (i == 0) {
                Marker marker = new Marker(mapaViewAmpliado);
                marker.setPosition(listaGeoPoints.get(i));
                marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
                marker.setIcon(getResources().getDrawable(R.drawable.circle_green, null));
                mapaViewAmpliado.getOverlayManager().add(marker);
            } else if (i == listaGeoPoints.size() - 1) {
                Marker marker = new Marker(mapaViewAmpliado);
                marker.setPosition(listaGeoPoints.get(i));
                marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
                marker.setIcon(getResources().getDrawable(R.drawable.circle_red, null));
                mapaViewAmpliado.getOverlayManager().add(marker);
            } //else {
                /*if(i!=listaGeoPoints.size()-1) {
                    LatLng l1 = new LatLng(listaGeoPoints.get(i).getLatitude(), listaGeoPoints.get(i).getLongitude());
                    LatLng l2 = new LatLng(listaGeoPoints.get(i+1).getLatitude(), listaGeoPoints.get(i+1).getLongitude());

                    Location location = new Location("l1");
                    location.setLatitude(l1.latitude);
                    location.setLongitude(l1.longitude);

                    Location location2 = new Location("l2");
                    location2.setLatitude(l2.latitude);
                    location2.setLongitude(l2.longitude);

                    float rotation = (float) SphericalUtil.computeHeading(l1,l2);

                    Marker marker = new Marker(mapaViewAmpliado);
                    marker.setPosition(listaGeoPoints.get(i));
                    marker.setAnchor(0.5f, 0.5f);
                    marker.setRotation(rotation);
                    marker.setFlat(true);
                    marker.setIcon(getResources().getDrawable(R.drawable.arrow, null));
                    mapaViewAmpliado.getOverlayManager().add(marker);
                }*/
            //}
        }


        mapaViewAmpliado.invalidate();
    }

    /*
     * Acción que nos permite centrar el mapa en relazión a la posición del usuario
     */
    @OnClick(R.id.imgCenter)
    @Override
    public void handleCenterMapa() {
        GeoPoint geoPoint = new GeoPoint(listaGeoPoints.get(0).getLatitude(), listaGeoPoints.get(0).getLongitude());
        mapController.setCenter(geoPoint);
    }

    private double brng(Location loc1, Location loc2){
        double brng = 0;

        double dLon = (loc2.getLongitude() - loc1.getLongitude());

        double y = Math.sin(dLon) * Math.cos(loc2.getLatitude());
        double x = Math.cos(loc1.getLatitude()) * Math.sin(loc2.getLatitude()) - Math.sin(loc1.getLatitude())
                * Math.cos(loc2.getLatitude()) * Math.cos(dLon);

        brng = Math.atan2(y, x);

        brng = Math.toDegrees(brng);
        brng = (brng + 360) % 360;
        brng = 360 - brng;

        return brng;
    }
}
