package com.uoc.christian.bttroute.MisRutas.Adapter;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.example.christian.myapplication.backend.rutaApi.model.Ruta;
import com.example.christian.myapplication.backend.tipoRutaApi.model.TipoRuta;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;
import dataUtils.Constants;
import dataUtils.DatosFiltros;
import dataUtils.DatosRuta;
import com.uoc.christian.bttroute.R;

import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

/**
 * Created by christian on 1/11/17.
 */

public class AdapterMisRutas extends RecyclerView.Adapter<AdapterMisRutas.ViewHolder> {

    private List<Ruta> listaRutasUsuario;
    private Context context;
    private OnItemClickListener onItemClickListener;
    private String tRuta = "";
    private String urlMapStatic = "";

    public AdapterMisRutas(List<Ruta> lista, Context context, OnItemClickListener onItemClickListener) {
        this.listaRutasUsuario = lista;
        this.context = context;
        this.onItemClickListener = onItemClickListener;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup viewGroup, int i) {
        View view = LayoutInflater.from(viewGroup.getContext())
                .inflate(R.layout.item_lista_mis_rutas, viewGroup, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(final ViewHolder viewHolder, int i) {
        final Ruta ruta = listaRutasUsuario.get(i);
        urlMapStatic = "https://www.mapquestapi.com/staticmap/v5/map?key=" + Constants.APIKeyMapQuest + "&shape=cmp|enc:" + ruta.getGeoPointsCodificados()+ "&size=500,300";

        viewHolder.textTituloRuta.setText(ruta.getTituloRuta());
        viewHolder.textDatoDistancia.setText(String.valueOf(ruta.getDistancia()) + " Km");
        viewHolder.textDatoTiempo.setText(ruta.getTiempo());
        for (TipoRuta tipoRuta : DatosFiltros.listaTipoRuta) {
            if (tipoRuta.getId().toString().equals(ruta.getIdTipoRuta().toString())) {
                tRuta = tipoRuta.getDescripcion();
            }
        }
        viewHolder.textDatoTipoRuta.setText(tRuta);

        Picasso.with(context).cancelRequest(viewHolder.openmapview);
        Picasso.with(context)
                .load(urlMapStatic)
                .into(viewHolder.openmapview, new Callback() {
                    @Override
                    public void onSuccess() {
                        if(viewHolder.progressbarImage!=null && viewHolder.progressbarImage.getVisibility()==View.VISIBLE) {
                            viewHolder.progressbarImage.setVisibility(View.GONE);
                        }
                    }

                    @Override
                    public void onError() {
                        if(viewHolder.progressbarImage!=null && viewHolder.progressbarImage.getVisibility()==View.VISIBLE) {
                            viewHolder.progressbarImage.setVisibility(View.GONE);
                        }
                    }
                });

        viewHolder.setClickListener(ruta, onItemClickListener);
    }


    @Override
    public int getItemCount() {
        return listaRutasUsuario.size();
    }

    public void setListaRutasUsuario(List<Ruta> lista) {
        this.listaRutasUsuario=lista;
        notifyDataSetChanged();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {

        @BindView(R.id.textDatoTiempo)
        TextView textDatoTiempo;
        @BindView(R.id.textTituloRuta)
        TextView textTituloRuta;
        @BindView(R.id.textDatoTipoRuta)
        TextView textDatoTipoRuta;
        @BindView(R.id.textDatoDistancia)
        TextView textDatoDistancia;
        @BindView(R.id.openmapview)
        ImageView openmapview;
        @BindView(R.id.progressbarImage)
        ProgressBar progressbarImage;

        private View view;

        public ViewHolder(View itemView) {
            super(itemView);
            this.view = itemView;
            ButterKnife.bind(this, view);

        }

        private void setClickListener(final Ruta ruta, final OnItemClickListener listener) {
            view.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {

                    DatosRuta datosRuta = new DatosRuta();
                    datosRuta.setIdUsuario(ruta.getIdUsuario());
                    datosRuta.setTituloRuta(ruta.getTituloRuta());
                    datosRuta.setObservaciones(ruta.getObservaciones());
                    datosRuta.setIdDificultadFisica(ruta.getIdDificultadFisica());
                    datosRuta.setIdDificultadTecnica(ruta.getIdDificultadTecnica());
                    datosRuta.setTiempo(ruta.getTiempo());
                    datosRuta.setDistancia(ruta.getDistancia());
                    datosRuta.setDesnivelPositivo(ruta.getDesnivelPositivo());
                    datosRuta.setVelocidadMaxima(ruta.getVelocidadMaxima());
                    datosRuta.setVelocidadMedia(ruta.getVelocidadMedia());
                    datosRuta.setIdTipoRuta(ruta.getIdTipoRuta());
                    datosRuta.setCompartida(ruta.getCompartida());
                    datosRuta.setIdRuta(ruta.getId());
                    datosRuta.setGeoPointsCodificados(ruta.getGeoPointsCodificados());
                    datosRuta.setDesnivelAcumulado(ruta.getDesnivelAcumulado());
                    datosRuta.setDesnivelNegativo(ruta.getDesnivelNegativo());
                    datosRuta.setIdRutaOrigen(ruta.getIdRutaOriginal());
                    datosRuta.setUsuarioFacebook(ruta.getUsuarioFacebook());


                    listener.onItemClick(datosRuta);
                }
            });
        }
    }

}
