package com.uoc.christian.bttroute.MisRutas;


import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.support.v4.app.Fragment;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.Toast;

import com.example.christian.myapplication.backend.rutaApi.RutaApi;
import com.example.christian.myapplication.backend.rutaApi.model.Ruta;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import com.omadahealth.github.swipyrefreshlayout.library.SwipyRefreshLayout;
import com.omadahealth.github.swipyrefreshlayout.library.SwipyRefreshLayoutDirection;
import dataUtils.DatosRuta;
import dataUtils.SharedPrefsUtils;
import com.uoc.christian.bttroute.MisRutas.Adapter.AdapterMisRutas;
import com.uoc.christian.bttroute.MisRutas.Adapter.OnItemClickListener;
import com.uoc.christian.bttroute.PantallaInicial.PantallaInicialActivity;
import com.uoc.christian.bttroute.R;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;
import dataUtils.ConexionBackend;

/**
 * A simple {@link Fragment} subclass.
 */
public class MisRutasFragment extends Fragment implements MisRutasView, OnItemClickListener, SwipyRefreshLayout.OnRefreshListener {


    @BindView(R.id.frameLayoutSinDatos)
    FrameLayout frameLayoutSinDatos;
    @BindView(R.id.frameLayoutConDatos)
    FrameLayout frameLayoutConDatos;
    @BindView(R.id.btnNuevaRuta)
    Button btnNuevaRuta;
    @BindView(R.id.listaMisRutas)
    RecyclerView listaMisRutas;
    @BindView(R.id.progressbar)
    ProgressBar progressbar;
    @BindView(R.id.layoutConconexion)
    RelativeLayout layoutConconexion;
    @BindView(R.id.layoutSinConexion)
    RelativeLayout layoutSinConexion;
    @BindView(R.id.btnActualizar)
    Button btnActualizar;
    @BindView(R.id.swipeContainer)
    SwipyRefreshLayout swipeContainer;

    Unbinder unbinder;

    RutaApi servicio = null;
    private boolean isLoginFacebook;
    private AdapterMisRutas adapter;
    private boolean tenemosConexion = true;
    private boolean refreshSwipe = false;

    MisRutasPresenterImplement presenter;

    public MisRutasFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_mis_rutas, container, false);
        unbinder = ButterKnife.bind(this, view);

        swipeContainer.setOnRefreshListener(this);

        setupPresenter();
        setupRecyclerView();

        isLoginFacebook = SharedPrefsUtils.getBooleanPreference(getContext(), getString(R.string.key_isLoginFacebook), false);

        //Comprobamos si tenemos rutas del usuario ya cargadas.Sino realizamos una conexión a la base de datos.
        if (presenter.getListaMisRutas()!= null && presenter.getListaMisRutas().size() > 0) {
            frameLayoutConDatos.setVisibility(View.VISIBLE);
            frameLayoutSinDatos.setVisibility(View.GONE);
            setRutas(presenter.getListaMisRutas());
        } else {
            obtenerRutasUsuario();
        }

        Log.d("MisRutas", "onCreateView");

        return view;
    }

    @Override
    public void setupPresenter() {
        presenter = new MisRutasPresenterImplement(getActivity().getApplicationContext());
    }

    @OnClick(R.id.btnNuevaRuta)
    @Override
    public void handleNuevaRuta() {
        ((PantallaInicialActivity) getActivity()).irARegistroActividad();
    }

    @OnClick(R.id.btnActualizar)
    @Override
    public void handelActualizarDatos() {
        obtenerRutasUsuario();
    }

    /*
     * Obtenemos las rutas del usuario a través de forma asincrona
     */
    @Override
    public void obtenerRutasUsuario() {
        if(presenter.getListaMisRutas()!=null && presenter.getListaMisRutas().size()>0){
            presenter.getListaMisRutas().clear();
        }
        new obtenerRutas().execute(0);
    }

    /*
     * Mostramos o ocultamos la barra de progreso
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if (mostrar) {
            progressbar.setVisibility(View.VISIBLE);
        } else {
            progressbar.setVisibility(View.INVISIBLE);
        }
    }

    @Override
    public void setupRecyclerView() {
        setupAdapter();
        listaMisRutas.setLayoutManager(new LinearLayoutManager(getActivity()));
        listaMisRutas.setAdapter(adapter);
    }

    @Override
    public void setupAdapter() {
        adapter = new AdapterMisRutas(new ArrayList<Ruta>(), getContext(), this);
    }

    @Override
    public void setRutas(List<Ruta> rutas) {
        adapter.setListaRutasUsuario(rutas);
    }

    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getActivity().getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        unbinder.unbind();
    }

    /*
     * Accedemos al detalla de la ruta del usuario y desde esa pantalla podremos descargarla en archivo GPX
     */
    @Override
    public void onItemClick(DatosRuta datosRuta) {
        ((PantallaInicialActivity) getActivity()).mostrarDetalleRuta(datosRuta);
    }

    @Override
    public void onRefresh(SwipyRefreshLayoutDirection direction) {
        refreshSwipe = true;
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                new obtenerRutas().execute(presenter.getListaMisRutas().size());
            }
        },2000);
    }

    public class obtenerRutas extends AsyncTask<Integer, Void, List<Ruta>> {

        List<Ruta> list = new ArrayList<>();

        @Override
        protected void onPreExecute() {
            if(!refreshSwipe) {
                mostrarProgressbar(true);
            }
        }

        @Override
        protected List<Ruta> doInBackground(Integer... nextObjects) {
            if (presenter.comprobarConexion()) {
                if (servicio == null) {
                    RutaApi.Builder builder = new RutaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                            .setRootUrl(ConexionBackend.url);
                    servicio = builder.build();
                }

                try {
                    //Comprobamos si el ha realizado el login con facebook o su cuenta de usuario y obtenemos sus rutas creadas
                    if (isLoginFacebook) {
                        list = servicio.getByIdFacebook(SharedPrefsUtils.getStringPreference(getContext(), getString(R.string.key_idUsuarioFacebook)), nextObjects[0]).execute().getItems();
                    } else {
                        list = servicio.getById(SharedPrefsUtils.getLongPreference(getContext(), getString(R.string.key_idusuario), 0), nextObjects[0]).execute().getItems();
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }

                tenemosConexion = true;
            } else {
                tenemosConexion = false;
            }

            return list;
        }

        @Override
        protected void onPostExecute(List<Ruta> rutas) {


            if(!refreshSwipe) {
                if (progressbar != null && progressbar.getVisibility() == View.VISIBLE) {
                    mostrarProgressbar(false);
                }
            } else{
                swipeContainer.setRefreshing(false);
            }

            if (rutas != null && rutas.size() > 0) {
                if (frameLayoutConDatos != null && frameLayoutSinDatos != null) {
                    frameLayoutConDatos.setVisibility(View.VISIBLE);
                    frameLayoutSinDatos.setVisibility(View.GONE);

                    btnActualizar.setVisibility(View.GONE);
                    btnNuevaRuta.setVisibility(View.VISIBLE);

                    presenter.setListaMisRutas(rutas);

                    setRutas(presenter.getListaMisRutas());
                }

            } else {
                if(presenter.getListaMisRutas()==null) {
                    if (!tenemosConexion) {
                        frameLayoutConDatos.setVisibility(View.GONE);
                        frameLayoutSinDatos.setVisibility(View.VISIBLE);
                        layoutSinConexion.setVisibility(View.VISIBLE);
                        layoutConconexion.setVisibility(View.GONE);

                        btnNuevaRuta.setVisibility(View.GONE);
                        btnActualizar.setVisibility(View.VISIBLE);

                        mostrarToast(getString(R.string.sinConexion));
                    } else {
                        frameLayoutConDatos.setVisibility(View.GONE);
                        frameLayoutSinDatos.setVisibility(View.VISIBLE);
                        layoutConconexion.setVisibility(View.VISIBLE);
                        layoutSinConexion.setVisibility(View.GONE);

                        btnNuevaRuta.setVisibility(View.VISIBLE);
                        btnActualizar.setVisibility(View.GONE);
                    }
                }
            }

            refreshSwipe = false;
        }
    }

}
