package com.uoc.christian.bttroute.RecordarPassword;

import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.christian.myapplication.backend.usuarioApi.UsuarioApi;
import com.example.christian.myapplication.backend.usuarioApi.model.Usuario;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import com.uoc.christian.bttroute.R;

import java.io.IOException;

import com.crashlytics.android.Crashlytics;
import io.fabric.sdk.android.Fabric;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.ConexionBackend;
import dataUtils.EnvioGmail;

public class RecordarPasswordActivity extends AppCompatActivity implements RecordarPasswordView {

    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.editEntradaUsuario)
    EditText editEntradaUsuario;
    @BindView(R.id.btnAceptar)
    Button btnAceptar;
    @BindView(R.id.progressBar)
    ProgressBar progressBar;

    RecordarPasswordPresenterImplement presenter;
    //Variable para guardar la nueva clave
    String nuevaClave = "";
    //Servicio
    UsuarioApi servicio;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_recordar_password);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        setupPresenter();
        setupActionbar();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch (item.getItemId()){
            case android.R.id.home:
                finish();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        finish();
    }

    @Override
    public void setupPresenter() {
        presenter = new RecordarPasswordPresenterImplement(this, getApplicationContext());
    }

    /*
     * Validamos y comprobamos el usuario
     */
    @OnClick(R.id.btnAceptar)
    @Override
    public void handleAceptar() {
        if(presenter.comprobarConexion()) {
            if(!editEntradaUsuario.getText().toString().isEmpty()) {
                new comprobarUsuarioYActualizar().execute(editEntradaUsuario.getText().toString());
            }else{
                mostrarToast(getResources().getString(R.string.recordarPassword_faltaUsuario));
            }
        }else{
            mostrarToast(getResources().getString(R.string.sinConexion));
        }
    }

    /*
     * Mostramos un toast con el mensaje recibido
     */
    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Mostramos o ocultamos la barra de progreso
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if(mostrar){
            progressBar.setVisibility(View.VISIBLE);
        }else{
            progressBar.setVisibility(View.INVISIBLE);
        }
    }

    /*
     * Bloquear campos
     */
    @Override
    public void bloquearCampos() {
        editEntradaUsuario.setEnabled(false);
        btnAceptar.setEnabled(false);
    }

    /*
     * Desbloquear campos
     */
    @Override
    public void desbloquearCampos() {
        editEntradaUsuario.setEnabled(true);
        btnAceptar.setEnabled(true);
    }

    /*
     * Seteamos la actionbar
     */
    private void setupActionbar(){
        appbar.setTitle(getResources().getString(R.string.app_name));
        appbar.setSubtitle(getResources().getString(R.string.recordar_titulo));
        setSupportActionBar(appbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }

    protected class comprobarUsuarioYActualizar extends AsyncTask<String, Void, Usuario>{

        @Override
        protected void onPreExecute(){
            mostrarProgressbar(true);
            bloquearCampos();
        }

        @Override
        protected Usuario doInBackground(String... values) {
            Usuario usuario = null;
            if(servicio == null){
                UsuarioApi.Builder builder = new UsuarioApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicio = builder.build();
            }

            try {
                //Comprobamos si el usuario introducido existe
                usuario = servicio.checkExist(values[0]).execute();
                if(usuario!=null){
                    //Si existe, actualizamos la clave del usuario en la base de datos
                    nuevaClave = presenter.crearClaveAleatoria();
                    servicio.updatePassword(usuario.getUser(), nuevaClave).execute();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            return usuario;
        }

        @Override
        protected void onPostExecute(Usuario usuario){
            if(usuario!=null){
                //Enviamos al usuario un mais
                EnvioGmail envio = new EnvioGmail(getApplicationContext(),usuario.getEmail(), "Nueva contraseña", "La nueva contraseña es: " + nuevaClave);
                envio.execute();

                mostrarProgressbar(false);

                Toast.makeText(getApplicationContext(), getResources().getString(R.string.recordarPassword_mailEnviado), Toast.LENGTH_LONG).show();

                finish();
            }else{
                mostrarProgressbar(false);
                mostrarToast(getResources().getString(R.string.recordarPassword_errorUsuario));
            }

            desbloquearCampos();
        }
    }

    /*protected class enviarEmail extends AsyncTask<String[], Void, Void> {


        @Override
        protected Void doInBackground(String[]... values) {

            return null;
        }

        @Override
        protected void onPostExecute(Void result){
            if(progressBar.getVisibility()==View.VISIBLE){
                mostrarProgressbar(false);
            }
        }
    }*/
}
