package com.uoc.christian.bttroute.RegistroActividad;

import android.Manifest;
import android.app.Activity;
import android.app.PendingIntent;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.location.GnssStatus;
import android.location.GpsSatellite;
import android.location.GpsStatus;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Build;
import android.os.Bundle;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.support.annotation.RequiresApi;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.widget.Button;
import android.widget.Chronometer;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.crashlytics.android.Crashlytics;
import com.google.android.gms.common.api.GoogleApiClient;
import com.google.android.gms.location.ActivityRecognitionClient;
import com.uoc.christian.bttroute.R;
import com.uoc.christian.bttroute.ResultadosActividad.ResultadosActividadActivity;

import org.osmdroid.config.Configuration;
import org.osmdroid.util.GeoPoint;
import org.osmdroid.views.MapController;
import org.osmdroid.views.MapView;
import org.osmdroid.views.overlay.Marker;
import org.osmdroid.views.overlay.Polyline;
import org.osmdroid.views.overlay.compass.CompassOverlay;
import org.osmdroid.views.overlay.compass.InternalCompassOrientationProvider;
import org.osmdroid.views.overlay.gestures.RotationGestureOverlay;
import org.osmdroid.views.overlay.mylocation.DirectedLocationOverlay;

import java.math.BigDecimal;
import java.util.ArrayList;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.Constants;
import dataUtils.DatosRuta;
import dataUtils.SharedPrefsUtils;
import io.fabric.sdk.android.Fabric;

public class RegistroActividadActivity extends AppCompatActivity implements RegistroActividadView {

    @BindView(R.id.openmapview)
    MapView openmapview;
    @BindView(R.id.distancia)
    TextView distancia;
    @BindView(R.id.velocidad)
    TextView velocidad;
    @BindView(R.id.velocidadMedia)
    TextView velocidadMedia;
    @BindView(R.id.btnGrabar)
    Button btnGrabar;
    @BindView(R.id.LayoutBotonGrabar)
    RelativeLayout LayoutBotonGrabar;
    @BindView(R.id.btnPausa)
    Button btnPausa;
    @BindView(R.id.btnParar)
    Button btnParar;
    @BindView(R.id.btnContinuar)
    Button btnContinuar;
    @BindView(R.id.LayoutBotonesPararContinuar)
    RelativeLayout LayoutBotonesPararContinuar;
    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.chronometer)
    Chronometer chronometer;
    @BindView(R.id.textSenalGPS)
    TextView textSenalGPS;
    @BindView(R.id.layoutGPSMensaje)
    RelativeLayout layoutGPSMensaje;
    @BindView(R.id.imgSetCenter)
    ImageView imgSetCenter;

    RegistroActividadPresenterImplement presenter;

    final static int PERMISSION_ALL = 1;
    final static String[] PERMISSIONS = {Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION};


    //Controlador del mapa
    MapController mapController;

    private LocationManager locationManager;
    private LocationListener locationListener;
    private int numVecesMostrado = 0;
    private GnssStatus.Callback mGnssStatusCallback;

    //Flag para saber si debemos pintar en el mapa o no
    private boolean guardarDatos;

    private long lastPause;

    private ArrayList<GeoPoint> pointsRutaCargada;

    private boolean rutaNueva = false;
    private int tiempoSegundos = 0;
    private float azimuthAngleSpeed = 0.0f;

    private Marker marker;
    private RotationGestureOverlay rotationGestureOverlay;
    private GeoPoint geoPointCenter;
    private DirectedLocationOverlay directedLocationOverlay;

    private boolean setCenter = true;
    private boolean localizacionActivada = true;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_registro_actividad);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        //important! set your user agent to prevent getting banned from the osm servers
        Configuration.getInstance().load(getApplicationContext(), PreferenceManager.getDefaultSharedPreferences(getApplicationContext()));
        setupActionbar();
        setupPresenter();
        setupMap();
        setupChronometer();

        Bundle bundle = getIntent().getExtras();
        rutaNueva = bundle.getBoolean(Constants.rutaNueva);
        if (!rutaNueva) {
            pointsRutaCargada = bundle.getParcelableArrayList(Constants.listaPoints);
            if (!bundle.getString(Constants.idRutaOrigen).isEmpty()) {
                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen), bundle.getString(Constants.idRutaOrigen));
            } else {
                SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen), null);
            }
            //marcamos la ruta sobre el mapa
            pintarRutaCargada(pointsRutaCargada);
        } else {
            SharedPrefsUtils.setStringPreference(getApplicationContext(), getString(R.string.key_idRutaOrigen), null);
        }

        /*
         * Listener para detectar si el usuario toca el mapa. Si lo toca, desactivamos el flag
         * que indica que el mapa se ha de centrar con el marker.
         */
        openmapview.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View view, MotionEvent motionEvent) {
                if (motionEvent.getAction() == MotionEvent.ACTION_DOWN) {
                    setCenter = false;
                } else if (motionEvent.getAction() == MotionEvent.ACTION_UP) {
                    setCenter = false;
                }
                return false;
            }
        });
    }

    @Override
    protected void onResume() {
        super.onResume();
        //This will refresh the osmdroid configuration on resumning.
        Configuration.getInstance().load(this, PreferenceManager.getDefaultSharedPreferences(this));
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        //Desactivamos el listener para que no se quede la localización activa
        if (locationListener != null) {
            locationManager.removeUpdates(locationListener);
        }
    }

    /*
     * Seteamos presentador
     */
    @Override
    public void setupPresenter() {
        this.presenter = new RegistroActividadPresenterImplement(this, new ArrayList<GeoPoint>());
    }

    @Override
    public Location getLastKnowPosition() {
        Location lastKnowLocation = null;
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        String locationProvider = LocationManager.NETWORK_PROVIDER;
        if (checkLocationPermission(this)) {
            lastKnowLocation = locationManager.getLastKnownLocation(locationProvider);
        }

        return lastKnowLocation;
    }

    /*
     * Seteamos mapa inicial
     */
    @Override
    public void setupMap() {
        mapController = (MapController) openmapview.getController();
        mapController.setZoom(17);
        openmapview.setMultiTouchControls(true);
        openmapview.setTilesScaledToDpi(true);
        openmapview.setMaxZoomLevel(30);
        openmapview.setFlingEnabled(true);

        directedLocationOverlay = new DirectedLocationOverlay(this);
        openmapview.getOverlays().add(directedLocationOverlay);

        //Ativamos el compás del mapa
        CompassOverlay mCompassOverlay = new CompassOverlay(getApplicationContext(),
                new InternalCompassOrientationProvider(getApplicationContext()),
                openmapview);
        mCompassOverlay.enableCompass();
        openmapview.getOverlays().add(mCompassOverlay);

        //Activamos la rotación del mapa
        rotationGestureOverlay = new RotationGestureOverlay(openmapview);
        rotationGestureOverlay.setEnabled(true);
        openmapview.getOverlays().add(rotationGestureOverlay);

        //Recuperamos la última posción
        Location location = getLastKnowPosition();

        marker = new Marker(openmapview);

        if (location != null) {
            GeoPoint geoPoint = new GeoPoint(location.getLatitude(), location.getLongitude());
            geoPointCenter = geoPoint;
            //Centramos la posición en el mapa y mostramos el marker
            mapController.setCenter(geoPoint);
            marker.setIcon(getResources().getDrawable(R.drawable.marker_blue, null));
            marker.setPosition(geoPoint);
            marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
            openmapview.getOverlays().add(marker);
        }
        openmapview.invalidate();

        iniciarLocationManager();
    }

    /*
     * Seteamos el cronómetro
     */
    @Override
    public void setupChronometer() {
        chronometer.setText("0:00:00");
        chronometer.setOnChronometerTickListener(new Chronometer.OnChronometerTickListener() {
            @Override
            public void onChronometerTick(Chronometer chronometer) {
                long time = SystemClock.elapsedRealtime() - chronometer.getBase();
                int h = (int) (time / 3600000);
                int m = (int) (time - h * 3600000) / 60000;
                int s = (int) (time - h * 3600000 - m * 60000) / 1000;
                //String hh = h < 10 ? "0"+h: h+"";
                String mm = m < 10 ? "0" + m : m + "";
                String ss = s < 10 ? "0" + s : s + "";
                chronometer.setText(h + ":" + mm + ":" + ss);
            }
        });
    }

    @OnClick(R.id.imgSetCenter)
    @Override
    public void handleSetCenter() {
        if (geoPointCenter != null) {
            openmapview.getController().animateTo(geoPointCenter);
            setCenter = true;
        }
    }

    /*
     * Iniciamos la grabación de la ruta
     */
    @OnClick(R.id.btnGrabar)
    @Override
    public void handleGrabar() {
        if (localizacionActivada) {
            btnPausa.setVisibility(View.VISIBLE);
            btnGrabar.setVisibility(View.GONE);
            chronometer.setBase(SystemClock.elapsedRealtime());

            chronometer.start();

            guardarDatos = true;

            setCenter = true;
        } else {
            activarLocalizacion();
        }
    }

    /*
     * Pausamos la grabación de la ruta
     */
    @OnClick(R.id.btnPausa)
    @Override
    public void handlePausa() {
        btnPausa.setVisibility(View.GONE);
        LayoutBotonGrabar.setVisibility(View.GONE);
        LayoutBotonesPararContinuar.setVisibility(View.VISIBLE);

        lastPause = SystemClock.elapsedRealtime();

        chronometer.stop();

        guardarDatos = false;

        velocidad.setText("0");

        setCenter = true;
    }

    /*
     * Continuamos la grabación de la ruta
     */
    @OnClick(R.id.btnContinuar)
    @Override
    public void handleContinuar() {
        btnPausa.setVisibility(View.VISIBLE);
        LayoutBotonesPararContinuar.setVisibility(View.GONE);
        LayoutBotonGrabar.setVisibility(View.VISIBLE);

        chronometer.setBase(chronometer.getBase() + SystemClock.elapsedRealtime() - lastPause);

        chronometer.start();

        guardarDatos = true;

        setCenter = true;
    }

    /*
     * Detenemos la grabación de la ruta
     */
    @OnClick(R.id.btnParar)
    @Override
    public void handleStop() {
        LayoutBotonesPararContinuar.setVisibility(View.GONE);
        LayoutBotonGrabar.setVisibility(View.VISIBLE);
        btnGrabar.setVisibility(View.VISIBLE);
        btnPausa.setVisibility(View.GONE);

        showFinalizarGrabarRutaDialog();

    }


    /*
     * Mostramos un mensaje toast
     */
    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getApplicationContext(), mensaje, Toast.LENGTH_SHORT).show();
    }

    /*
     * Mostramos mensaje
     */
    @Override
    public void showExitGrabarRutaDialog() {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(this, R.style.AlertDialogCustom);
        builder.setTitle(getString(R.string.registroActividad_titulo_mensaje_cerrar))
                .setMessage(R.string.registroActividad_mensaje_cerrar)
                .setPositiveButton(R.string.buttons_alert_aceptar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        finish();
                    }
                })
                .setNegativeButton(R.string.buttons_alert_cancelar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // do nothing
                    }
                })
                .create()
                .show();
    }

    /*
     * Mostramos mensaje para preguntar si el usuario quiere finalizar la ruta
     */
    @Override
    public void showFinalizarGrabarRutaDialog() {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(this, R.style.AlertDialogCustom);
        builder.setTitle(getString(R.string.registroActividad_titulo_mensaje_cerrar))
                .setMessage(R.string.registroActividad_mensaje_finalizar)
                .setPositiveButton(R.string.buttons_alert_aceptar, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {
                        guardarDatos = false;

                        presenter.separarDesniveles();
                        DatosRuta datosRuta = new DatosRuta();
                        datosRuta.setDistancia(Float.parseFloat(distancia.getText().toString()));
                        datosRuta.setTiempo(chronometer.getText().toString());
                        datosRuta.setVelocidadMedia(Float.parseFloat(velocidadMedia.getText().toString()));
                        datosRuta.setVelocidadMaxima(presenter.getVelocidadMaxima());
                        datosRuta.setDesnivelPositivo(presenter.getDesnivelPositivo());
                        datosRuta.setGeoPointsCodificados(presenter.codificarGeopoints(presenter.getCoordenadas()));
                        datosRuta.setDesnivelNegativo(presenter.getDesnivelNegativo());
                        datosRuta.setDesnivelAcumulado(presenter.getDesnivelAcumulado());

                        if (locationListener != null) {
                            locationManager.removeUpdates(locationListener);
                            locationManager.removeGpsStatusListener(gpsStatus);
                            locationListener = null;
                        }

                        Intent intent = new Intent(RegistroActividadActivity.this, ResultadosActividadActivity.class);
                        intent.putExtra(Constants.datosRutaUsuario, datosRuta);
                        startActivity(intent);
                        finish();
                    }
                })
                .setNegativeButton(R.string.buttons_alert_cancelar, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface, int i) {

                    }
                })
                .create()
                .show();
    }

    /*
     * Iniciamos la localización del usuario con el GPS
     */
    @Override
    public void iniciarLocationManager() {
        locationListener = new MyLocationListener();
        locationManager = (LocationManager) getSystemService(Context.LOCATION_SERVICE);
        if (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {

            if (checkLocationPermission(this)) {
                String provider = locationManager.getBestProvider(presenter.getCriteria(), true);

                //Si la API es inferior a 24 utilizamos el listener de GpsStatus, si no GnssStatus
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                    mGnssStatusCallback = new GnssStatus.Callback() {
                        boolean gpsFix = false;
                        boolean mostrar = false;

                        @Override
                        public void onFirstFix(int ttffMillis) {
                            gpsFix = true;
                        }

                        @RequiresApi(api = Build.VERSION_CODES.N)
                        @Override
                        public void onSatelliteStatusChanged(GnssStatus status) {
                            if (status.getSatelliteCount() == 0) {
                                gpsFix = false;
                                mostrar = true;
                            } else {
                                gpsFix = true;
                                mostrar = true;
                            }

                            if (gpsFix) {
                                if (numVecesMostrado < 2 && mostrar) {
                                    mostrarEncontradoGPS();
                                    numVecesMostrado++;
                                } else {
                                    layoutGPSMensaje.setVisibility(View.GONE);
                                    mostrar = !mostrar;
                                }

                            } else {
                                mostrarBuscandoGPS();
                                numVecesMostrado = 0;
                                mostrar = true;
                                velocidad.setText("0");
                            }
                        }
                    };
                } else {
                    locationManager.addGpsStatusListener(gpsStatus);
                }

                locationManager.requestLocationUpdates(provider, 1000, 0, locationListener);
            }

            setCenter = true;
            localizacionActivada = true;
        } else {
            activarLocalizacion();
        }
    }

    /*
     * Seteamos el campo distancia
     */
    @Override
    public void setearDistancia(String dist) {
        distancia.setText(dist);
    }

    /*
     * Seteamos el campo velocidad media
     */
    @Override
    public void setearVelocidadMedia(String velMedia) {
        velocidadMedia.setText(velMedia);
    }

    /*
     * Seteamos el campo velocidad
     */
    @Override
    public void setearVelocidad(String vel) {
        velocidad.setText(vel);
    }

    /*
     * Marcamos la ruta cargada desde el archivo GPX
     */
    @Override
    public void pintarRutaCargada(final ArrayList<GeoPoint> listpoints) {


        Polyline polyline = new Polyline();
        polyline.setGeodesic(true);
        polyline.setPoints(listpoints);
        polyline.setWidth(15f);
        polyline.setColor(Color.BLUE);


        openmapview.getOverlayManager().add(polyline);

        //Añadimos las marcas de inicio y meta en la ruta
        for (int i = 0; i < listpoints.size(); i++) {
            if (i == 0) {
                Marker marker = new Marker(openmapview);
                marker.setPosition(listpoints.get(i));
                marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
                marker.setIcon(getResources().getDrawable(R.drawable.circle_green, null));
                openmapview.getOverlays().add(marker);
            } else if (i == listpoints.size() - 1) {
                Marker marker = new Marker(openmapview);
                marker.setPosition(listpoints.get(i));
                marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
                marker.setIcon(getResources().getDrawable(R.drawable.circle_red, null));
                openmapview.getOverlays().add(marker);
            }
        }
        openmapview.invalidate();
    }

    /*
     * Mostramos el mensaje para que el usuario active la localización
     */
    @Override
    public void activarLocalizacion() {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(this, R.style.AlertDialogCustom);
        builder.setTitle(getString(R.string.registroActividad_gps_title))
                .setMessage(R.string.registroActividad_gps_mensaje)
                .setPositiveButton(R.string.buttons_alert_aceptar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                        startActivity(intent);
                    }
                })
                .setNegativeButton(R.string.buttons_alert_cancelar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        localizacionActivada = false;
                    }
                })
                .create()
                .show();
    }

    /*
     * Cargamos el menú
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_grabar_ruta, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.menu_cerrar) {
            showExitGrabarRutaDialog();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void onBackPressed() {
        showExitGrabarRutaDialog();
    }

    /*
     * Seteamos la toolbar
     */
    private void setupActionbar() {
        appbar.setTitle(getString(R.string.app_name));
        appbar.setSubtitle(getString(R.string.registro_actividad_titulo));
        setSupportActionBar(appbar);
    }

    public class MyLocationListener implements LocationListener {

        @Override
        public void onLocationChanged(Location location) {

            if (location == null) {
                setearVelocidad("0");
                return;
            }

            GeoPoint point = new GeoPoint(location.getLatitude(), location.getLongitude());

            if (setCenter) {
                mapController.animateTo(point);
            }

            geoPointCenter = point;
            marker.setIcon(getResources().getDrawable(R.drawable.marker_blue, null));
            marker.setPosition(point);
            marker.setAnchor(Marker.ANCHOR_CENTER, Marker.ANCHOR_BOTTOM);
            openmapview.getOverlays().add(marker);

            //Solo guardamos los datos cuando se esta grabando
            if (guardarDatos) {

                //Obtenemos el tiempo en segundos.
                tiempoSegundos = presenter.getTiempoEnSegundos(chronometer.getText().toString());

                directedLocationOverlay.setEnabled(true);
                openmapview.setMapOrientation(presenter.calcularGrados(location));

                double velocidad = 0;


                velocidad = location.getSpeed() * 3.6;

                if (velocidad > 0.1) {
                    presenter.calcularDistancia(location);
                    presenter.guardarCoordenadasTemp(point);
                    presenter.guardarAltitud((int) location.getAltitude());
                    presenter.calcularVelocidadMedia(location, tiempoSegundos);

                    float vel = BigDecimal.valueOf(velocidad).setScale(1, BigDecimal.ROUND_HALF_UP).floatValue();
                    setearVelocidad(String.valueOf(vel));
                    presenter.guardarVelocidadMaxima(velocidad);

                    azimuthAngleSpeed = location.getBearing();
                    directedLocationOverlay.setBearing(azimuthAngleSpeed);

                    //Si se trata de una ruta nueva, si que pintamos el recorrido realizado
                    if (rutaNueva) {
                        Polyline polyline = new Polyline();
                        polyline.setWidth(20f);
                        polyline.setColor(R.color.colorLineaRuta);
                        polyline.setPoints(presenter.getCoordenadas());
                        polyline.setGeodesic(true);
                        openmapview.getOverlayManager().add(polyline);
                    }

                } else {
                    setearVelocidad("0");
                }
            }
        }

        @Override
        public void onStatusChanged(String s, int status, Bundle bundle) {
            Log.d("GPSStatusChanged", s.toString() + "int " + status);
        }

        @Override
        public void onProviderEnabled(String s) {
            Log.d("GPSProviderEnabled", s.toString());
        }

        @Override
        public void onProviderDisabled(String s) {
            Log.d("GPSProviderDisabled", s.toString());
        }
    }

    public static boolean checkLocationPermission(Activity activity) {
        if (ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED
                || ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_COARSE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            ActivityCompat.requestPermissions(activity, PERMISSIONS, PERMISSION_ALL);
            return false;
        }
        return true;
    }

    /*
     * GpsStatus actualmente está deprecated pero para versiones inferiores a la API 24 todavía se utiliza
     */
    public GpsStatus.Listener gpsStatus = new GpsStatus.Listener() {
        @Override
        public void onGpsStatusChanged(int event) {
            boolean gpsFix = false;
            boolean mostrar = false;
            switch (event) {
                case GpsStatus.GPS_EVENT_FIRST_FIX:
                    gpsFix = true;

                    break;
                case GpsStatus.GPS_EVENT_SATELLITE_STATUS:
                    if (checkLocationPermission(RegistroActividadActivity.this)) {
                        GpsStatus g = locationManager.getGpsStatus(null);
                        int usedStats = 0;
                        for (GpsSatellite sat : g.getSatellites()) {
                            if (sat.usedInFix()) {
                                usedStats++;
                            }
                        }
                        //Log.d("Satellites", String.valueOf(usedStats));

                        if (usedStats == 0) {
                            gpsFix = false;
                            mostrar = true;
                        } else if (usedStats >= 1) {
                            gpsFix = true;
                            mostrar = true;
                        }

                        if (gpsFix) {
                            if (numVecesMostrado < 2 && mostrar) {
                                mostrarEncontradoGPS();
                                numVecesMostrado++;
                            } else {
                                layoutGPSMensaje.setVisibility(View.GONE);
                                mostrar = !mostrar;
                            }
                        } else {
                            mostrarBuscandoGPS();
                            numVecesMostrado = 0;
                            mostrar = true;
                            setearVelocidad("0");
                        }
                    }

                    break;

            }
        }
    };

    /*
     * Mostramos el mensaje cuando no tengamos señal GPS
     */
    private void mostrarBuscandoGPS() {
        layoutGPSMensaje.setVisibility(View.VISIBLE);
        layoutGPSMensaje.setBackgroundColor(Color.BLACK);
        textSenalGPS.setText(getString(R.string.registro_actividad_mensajeGPSBuscando));
    }

    /*
     * Mostramos el mensaje cuando tengamos señal GPS
     */
    private void mostrarEncontradoGPS() {
        layoutGPSMensaje.setVisibility(View.VISIBLE);
        layoutGPSMensaje.setBackgroundColor(ContextCompat.getColor(getApplicationContext(), R.color.colorGPSEncontrado));
        textSenalGPS.setText(getString(R.string.registro_actividad_mensajeGPSEncontrado));
    }


}
