package com.uoc.christian.bttroute.RegistroUsuario;

import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.christian.myapplication.backend.usuarioApi.UsuarioApi;
import com.example.christian.myapplication.backend.usuarioApi.model.Usuario;

import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import dataUtils.Constants;
import com.uoc.christian.bttroute.Login.LoginActivity;
import com.uoc.christian.bttroute.R;

import com.crashlytics.android.Crashlytics;
import io.fabric.sdk.android.Fabric;

import java.io.IOException;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.ConexionBackend;

public class RegistroActivity extends AppCompatActivity implements RegistroView {

    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.editEntradaUsuario)
    EditText editEntradaUsuario;
    @BindView(R.id.editEntradaPassword)
    EditText editEntradaPassword;
    @BindView(R.id.editEntradaRepitePassword)
    EditText editEntradaRepitePassword;
    @BindView(R.id.editEntradaEmail)
    EditText editEntradaEmail;
    @BindView(R.id.btnRegistrar)
    Button btnRegistrar;
    @BindView(R.id.progressBar)
    ProgressBar progressBar;

    RegistroPresenterImplement presenter;
    //Servicio
    private UsuarioApi servicio = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_registro);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        setupPresenter();
        setupActionbar();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch (item.getItemId()){
            case android.R.id.home:
                Intent intent = new Intent(RegistroActivity.this, LoginActivity.class);
                setResult(RESULT_CANCELED, intent);
                finish();
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public void setupPresenter() {
        presenter = new RegistroPresenterImplement(this, getApplicationContext());
    }

    /*
     * Método para registrar el usuario
     */
    @OnClick(R.id.btnRegistrar)
    @Override
    public void handleRegistrar() {
        if(presenter.comprobarConexion()) {
            //Comprobamos si todos los datos están completados
            if (presenter.comprobarCampos(editEntradaUsuario.getText().toString(), editEntradaPassword.getText().toString(), editEntradaRepitePassword.getText().toString(), editEntradaEmail.getText().toString())) {
                //Comprobamos si las contraseñas son iguales
                if (presenter.comprobarPasswords(editEntradaPassword.getText().toString(), editEntradaRepitePassword.getText().toString())) {
                    if (presenter.comprobarMailValido(editEntradaEmail.getText().toString())) {
                        String[] values = {editEntradaUsuario.getText().toString(), editEntradaPassword.getText().toString(), editEntradaEmail.getText().toString()};
                        new registrarUsuario().execute(values);
                    } else {
                        mostrarToastError(getResources().getString(R.string.registro_errorEmail));
                    }
                } else {
                    mostrarToastError(getResources().getString(R.string.registro_passwordsDiferentes));
                }
            } else {
                mostrarToastError(getResources().getString(R.string.registro_faltanDatos));
            }
        }else{
            mostrarToastError(getResources().getString(R.string.sinConexion));
        }
    }

    /*
     * Mostramos mensajes de error en toast
     */
    @Override
    public void mostrarToastError(String mensaje) {
        Toast.makeText(getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Mostramos o ocultamos el progressbar
     */
    @Override
    public void mostrarProgressbar(boolean mostar) {
        if(mostar){
            progressBar.setVisibility(View.VISIBLE);
        } else {
            progressBar.setVisibility(View.INVISIBLE);
        }
    }

    /*
     * Bloqueamos los campos
     */
    @Override
    public void bloquearCampos() {
        editEntradaUsuario.setEnabled(false);
        editEntradaPassword.setEnabled(false);
        editEntradaRepitePassword.setEnabled(false);
        editEntradaEmail.setEnabled(false);
    }

    /*
     * Desbloqueamos los campos
     */
    @Override
    public void desbloquearCampos() {
        editEntradaUsuario.setEnabled(true);
        editEntradaPassword.setEnabled(true);
        editEntradaRepitePassword.setEnabled(true);
        editEntradaEmail.setEnabled(true);
    }

    @Override
    public void onBackPressed() {
        Intent intent = new Intent(RegistroActivity.this, LoginActivity.class);
        setResult(RESULT_CANCELED, intent);
        finish();
    }

    private void setupActionbar(){
        appbar.setTitle(R.string.app_name);
        appbar.setSubtitle(R.string.registro_titulo);
        setSupportActionBar(appbar);
        getSupportActionBar().setDisplayHomeAsUpEnabled(true);
    }

    /*
     * Método asincrono para conectarnos al ConexionBackend y registrar el usuario
     */
    public class registrarUsuario extends AsyncTask<String[], Void, Usuario>{

        @Override
        protected void onPreExecute(){
            mostrarProgressbar(true);
            bloquearCampos();
        }

        @Override
        protected Usuario doInBackground(String[]... params) {
            Usuario usuario = null;

            String mUsuario = "";
            String mPassword = "";
            String mEmail = "";

            for (String[] s : params) {
                mUsuario = s[0];
                mPassword = s[1];
                mEmail = s[2];
            }

            if (servicio == null) {
                UsuarioApi.Builder builder = new UsuarioApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                        .setRootUrl(ConexionBackend.url);
                servicio = builder.build();
            }

            try {
                //Comprobamos si el nombre de usuario introducido ya existe en el sistema
                if (servicio.checkExist(mUsuario).execute() == null) {

                    usuario = new Usuario();
                    usuario.setUser(mUsuario);
                    usuario.setPassword(mPassword);
                    usuario.setEmail(mEmail);

                    servicio.insert(usuario).execute();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            return usuario;
        }

        @Override
        protected void onPostExecute(Usuario usuario){
            if(progressBar.getVisibility() == View.VISIBLE){
                mostrarProgressbar(false);
            }

            desbloquearCampos();

            if (usuario != null) {
                Intent intent = new Intent(RegistroActivity.this, LoginActivity.class);
                intent.putExtra(Constants.usuario, usuario.getUser());
                intent.putExtra(Constants.password, usuario.getPassword());
                setResult(Activity.RESULT_OK, intent);
                finish();
            } else {
                mostrarToastError(getResources().getString(R.string.registro_UsuarioExistente));
            }
        }
    }
}
