package com.uoc.christian.bttroute.ResultadosActividad;

import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.crashlytics.android.Crashlytics;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;
import com.uoc.christian.bttroute.GuardarRuta.GuardarRutaActivity;
import com.uoc.christian.bttroute.R;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import dataUtils.Constants;
import dataUtils.DatosRuta;
import io.fabric.sdk.android.Fabric;

public class ResultadosActividadActivity extends AppCompatActivity implements ResultadosActivityView {

    @BindView(R.id.appbar)
    Toolbar appbar;
    @BindView(R.id.mapView)
    ImageView mapView;
    @BindView(R.id.textResultadoTiempo)
    TextView textResultadoTiempo;
    @BindView(R.id.textResultadoDistancia)
    TextView textResultadoDistancia;
    @BindView(R.id.textResultadoVelMedia)
    TextView textResultadoVelMedia;
    @BindView(R.id.textResultadoVelMaxima)
    TextView textResultadoVelMaxima;
    @BindView(R.id.textResultadoDesnivel)
    TextView textResultadoDesnivel;
    @BindView(R.id.btnGuardarRuta)
    Button btnGuardarRuta;
    @BindView(R.id.textResultDesnivelNegativo)
    TextView textResultDesnivelNegativo;
    @BindView(R.id.textResultDesnivelAcumulado)
    TextView textResultDesnivelAcumulado;
    @BindView(R.id.progressbar)
    ProgressBar progressbar;

    private DatosRuta datosRuta = null;
    private ResultadosActivityPresenterImplement presenter = null;
    private String urlMapStatic = "";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_resultados_actividad);
        ButterKnife.bind(this);
        Fabric.with(this, new Crashlytics());
        setupActionbar();

        Bundle bundle = getIntent().getExtras();
        //Obtenemos los datos de la ruta
        datosRuta = (DatosRuta) bundle.getSerializable(Constants.datosRutaUsuario);
        //Cargamos el mapa de la ruta realizada
        obtenerMapa(datosRuta.getGeoPointsCodificados());
        //Seteamos presentador
        setupPresenter();
        setDesnivelNegativo();
        setDesnivelAcumulado();
        setDesnivelPositivo();
        setDistancia();
        setTiempo();
        setVelocidadMaxima();
        setVelocidadMedia();
    }

    /*
     * Cargamos el menú
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_resultados_ruta, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();
        if (id == R.id.menu_ruta_resultado_borrar) {
            showCancelarGrabar();
            return true;
        }
        return super.onOptionsItemSelected(item);
    }

    /*
     * Inicialización presenter
     */
    @Override
    public void setupPresenter() {
        presenter = new ResultadosActivityPresenterImplement(this, datosRuta);
    }

    /*
     * Setear actionbar
     */
    @Override
    public void setupActionbar() {
        appbar.setTitle(getString(R.string.app_name));
        appbar.setSubtitle(getString(R.string.resultados_titulo));
        setSupportActionBar(appbar);
    }

    /*
     * Acción guardar ruta
     */
    @OnClick(R.id.btnGuardarRuta)
    @Override
    public void handleGuardarRuta() {
        Intent intent = new Intent(ResultadosActividadActivity.this, GuardarRutaActivity.class);
        intent.putExtra(Constants.datosRutaUsuario, datosRuta);
        startActivity(intent);
    }

    /*
     * Acción mostrar dialog
     */
    @Override
    public void showCancelarGrabar() {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(this, R.style.AlertDialogCustom);
        builder.setTitle(getString(R.string.guardar_ruta_title_eliminarRuta))
                .setMessage(R.string.guardar_ruta_mensaje_eliminarRuta)
                .setPositiveButton(R.string.buttons_alert_aceptar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        finish();
                    }
                })
                .setNegativeButton(R.string.buttons_alert_cancelar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        // do nothing
                    }
                })
                .create()
                .show();
    }

    /*
     * Setear distancia
     */
    @Override
    public void setDistancia() {
        textResultadoDistancia.setText(presenter.getDistancia() + " Km");
    }

    /*
     * Setear tiempo
     */
    @Override
    public void setTiempo() {
        textResultadoTiempo.setText(presenter.getTiempo());
    }

    /*
     * Setear velocidad media
     */
    @Override
    public void setVelocidadMedia() {
        textResultadoVelMedia.setText(presenter.getVelocidadMedia() + " Km/h");
    }

    /*
     * Setear velocidad máxima
     */
    @Override
    public void setVelocidadMaxima() {
        textResultadoVelMaxima.setText(presenter.getVelocidadMaxima() + " Km/h");
    }

    /*
     * Setear desnivel
     */
    @Override
    public void setDesnivelPositivo() {
        textResultadoDesnivel.setText(presenter.getDesnivelPositivo() + " m");
    }

    @Override
    public void setDesnivelNegativo() {
        textResultDesnivelNegativo.setText(presenter.getDesnivelNegativo() + " m");
    }

    @Override
    public void setDesnivelAcumulado() {
        textResultDesnivelAcumulado.setText(presenter.getDesnivelAcumulado() + " m");
    }

    /*
     * A través de la url indicada y pasando los geopoints codificados obtenemos una imagen de la ruta.
     */
    @Override
    public void obtenerMapa(String geoPointsCodificados) {
        urlMapStatic = "https://www.mapquestapi.com/staticmap/v5/map?key=" + Constants.APIKeyMapQuest + "&shape=fill:333333|weight:5|cmp|enc:" + geoPointsCodificados + "&size=500,300";
        Picasso.with(getApplicationContext()).cancelRequest(mapView);
        Picasso.with(getApplicationContext())
                .load(urlMapStatic)
                .into(mapView, new Callback() {
                    @Override
                    public void onSuccess() {
                        if(progressbar!=null && progressbar.getVisibility()== View.VISIBLE) {
                            progressbar.setVisibility(View.GONE);
                        }
                    }

                    @Override
                    public void onError() {
                        if(progressbar!=null && progressbar.getVisibility()==View.VISIBLE) {
                            progressbar.setVisibility(View.GONE);
                        }
                    }
                });
    }

    /*
     * Acción botón volver
     */
    @Override
    public void onBackPressed() {
        showCancelarGrabar();
    }
}
