package com.uoc.christian.bttroute.RutasServidorBusqueda;


import android.Manifest;
import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.provider.Settings;
import android.support.v4.app.ActivityCompat;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.example.christian.myapplication.backend.rutaApi.RutaApi;
import com.example.christian.myapplication.backend.rutaApi.model.Ruta;
import com.google.api.client.extensions.android.http.AndroidHttp;
import com.google.api.client.extensions.android.json.AndroidJsonFactory;
import com.omadahealth.github.swipyrefreshlayout.library.SwipyRefreshLayout;
import com.omadahealth.github.swipyrefreshlayout.library.SwipyRefreshLayoutDirection;
import dataUtils.Constants;
import dataUtils.DatosFiltros;
import dataUtils.DatosPosicion;
import dataUtils.DatosRuta;
import com.uoc.christian.bttroute.PantallaInicial.PantallaInicialActivity;
import com.uoc.christian.bttroute.R;
import com.uoc.christian.bttroute.RutasServidorBusqueda.Adapter.OnItemClickListener;
import com.uoc.christian.bttroute.RutasServidorBusqueda.Adapter.RutasServidorAdapter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;
import butterknife.OnClick;
import butterknife.Unbinder;
import dataUtils.ConexionBackend;

/**
 * A simple {@link Fragment} subclass.
 */
public class RutasServidorFragment extends Fragment implements RutasServidorView, OnItemClickListener, SwipyRefreshLayout.OnRefreshListener {

    @BindView(R.id.listaRutasServidor)
    RecyclerView listaRutasServidor;
    @BindView(R.id.progressbar)
    ProgressBar progressbar;
    @BindView(R.id.layoutSinDatos)
    RelativeLayout layoutSinDatos;
    @BindView(R.id.layoutConDatos)
    RelativeLayout layoutConDatos;
    @BindView(R.id.btnActualizar)
    Button btnActualizar;
    @BindView(R.id.textSinResultados)
    TextView textSinResultados;
    @BindView(R.id.swipeContainer)
    SwipyRefreshLayout swipeContainer;

    Unbinder unbinder;


    private RutaApi servicioRuta;
    private RutasServidorAdapter adapter;
    private LocationManager locationManager;
    RutasServidorPresenterImplement presenter;
    private boolean tenemosConexion = true;
    private LocationListener locationListener;
    private boolean refreshSwipe = false;


    final static int PERMISSION_ALL = 1;
    final static String[] PERMISSIONS = {Manifest.permission.ACCESS_COARSE_LOCATION, Manifest.permission.ACCESS_FINE_LOCATION};

    public RutasServidorFragment() {
        // Required empty public constructor
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        // Inflate the layout for this fragment
        View view = inflater.inflate(R.layout.fragment_buscar_ruta, container, false);
        unbinder = ButterKnife.bind(this, view);

        swipeContainer.setOnRefreshListener(this);

        setupPresenter();

        obtenerPosicionUsuario();

        setRecyclerview();
        //Comprobamos si el usuario ya había hecho una búsqueda y cargamos las rutas buscadas.
        if (presenter.getRutasServidor() != null && presenter.getRutasServidor().size() > 0) {
            setRecyclerview();
            setRutas(presenter.getRutasServidor());
        } else {
            //Si es la primera vez que accedemos a la lista de rutas del servidor y no habiamos
            //realizado ninguna búsqueda, obtenemos las rutas cercanas a nuestra localización
            new obtenerRutasPorCercaniaUsuario().execute(0);
        }

        return view;
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        unbinder.unbind();
    }

    /*
     * Seteamos el presentador
     */
    @Override
    public void setupPresenter() {
        presenter = new RutasServidorPresenterImplement(getActivity().getApplicationContext());
    }

    /*
     * Llamamos al método que se encarga de obtener las rutas con los parámetros indicados
     */
    @Override
    public void actualizarLista() {

        layoutConDatos.setVisibility(View.GONE);
        layoutSinDatos.setVisibility(View.GONE);
        refreshSwipe = true;

        //Seteamos el recyclerview de la lista
        setRecyclerview();

        //Si teniamos elementos en la lista, limpiamos la lista y los volvemos a mostrar todos
        if(presenter.getRutasServidor()!=null && presenter.getRutasServidor().size()>0) {
            presenter.getRutasServidor().clear();
        }

        //Comprobamos si el usuario a aplicado filtros de búsqueda
        if (presenter.getIsFiltroAplicados()) {
            //Comprobamos si tenemos activada la localización
            if (locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER) || locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                //Si no tenemos localización del usuario, obtenemos la última localización
                if (presenter.getLatitud() == 0.0 && presenter.getLongitude() == 0.0) {
                    obtenerPosicionUsuario();
                }
                //Obtenemos las rutas cercanas
                new obtenerRutasServidorConParametros().execute(0);
            } else {
                //Si no está ativada la localización, solicitamos al usuario que la active
                activarLocalizacion();
            }
        } else {
            //Obtenemos la localización del usuario
            obtenerPosicionUsuario();
            //El usuario ha limpiado los filtros y volvemos a mostrar las rutas cercanas a nuestra
            //localización
            new obtenerRutasPorCercaniaUsuario().execute(0);
        }
    }

    /*
     * Mostramos o ocultamos la barra de progreso
     */
    @Override
    public void mostrarProgressbar(boolean mostrar) {
        if (mostrar) {
            progressbar.setVisibility(View.VISIBLE);
        } else {
            progressbar.setVisibility(View.INVISIBLE);
        }
    }

    /*
     * Seteamos el recyclerview
     */
    @Override
    public void setRecyclerview() {
        setupAdapter();
        listaRutasServidor.setLayoutManager(new LinearLayoutManager(getActivity()));
        listaRutasServidor.setAdapter(adapter);
    }

    /*
     * Seteamos el adapter del recyclerview
     */
    @Override
    public void setupAdapter() {
        adapter = new RutasServidorAdapter(new ArrayList<Ruta>(), this, getContext());
    }

    /*
     * cargamos las rutas en la lista
     */
    @Override
    public void setRutas(List<Ruta> rutas) {
        adapter.setRutas(rutas);
    }

    /*
     * Obtenemos la posición inicial que nos servirá para obtener las rutas cercanas a nuestra
     * localicación.
     */
    @Override
    public void obtenerPosicionUsuario() {
        Location lastKnownLocation = null;
        locationManager = (LocationManager) getActivity().getSystemService(getContext().LOCATION_SERVICE);

        if (checkLocationPermission(getActivity())) {
            if (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER) || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                if (locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER))
                    lastKnownLocation = locationManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
                else
                    lastKnownLocation = locationManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);

                if (presenter.getLatitud() == 0.0 && presenter.getLongitude() == 0.0) {
                    if (lastKnownLocation != null) {
                        presenter.setLocationUsuario(lastKnownLocation.getLatitude(), lastKnownLocation.getLongitude());
                    } else {
                        obtenerNuevaPosicion();
                    }
                } else {
                    //Calculamos la distancia con la anterior posición

                    //Posición incial
                    Location locationInicial = new Location("Inicial");
                    locationInicial.setLatitude(presenter.getLatitud());
                    locationInicial.setLongitude(presenter.getLongitude());
                    //Posición nueva
                    Location locationNueva = new Location("Nueva");
                    locationNueva.setLatitude(lastKnownLocation.getLatitude());
                    locationNueva.setLongitude(lastKnownLocation.getLongitude());

                    double distanceTo = locationInicial.distanceTo(locationNueva);
                    int distancia = (int) distanceTo; // distancia en metros

                    if (distancia > 1000) {
                        presenter.setLocationUsuario(locationNueva.getLatitude(), locationNueva.getLongitude());
                    }
                }
            } else {
                activarLocalizacion();
            }
        }
    }

    /*
     * Solicitamos al usuario que active la localización del usuario
     */
    @Override
    public void activarLocalizacion() {
        AlertDialog.Builder builder;
        builder = new AlertDialog.Builder(getActivity(), R.style.AlertDialogCustom);
        builder.setTitle(getString(R.string.registroActividad_gps_title))
                .setMessage(R.string.registroActividad_gps_mensaje)
                .setPositiveButton(R.string.buttons_alert_aceptar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                        startActivity(intent);
                    }
                })
                .setNegativeButton(R.string.buttons_alert_cancelar, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int which) {
                        presenter.setLocationUsuario(0.0, 0.0);
                    }
                })
                .create()
                .show();
    }

    /*
     * Acción botón actualizar que se muestra cuando se detecta que no se tiene conexión para que el usuario
     * pueda volver a intentar la búsqueda
     */
    @OnClick(R.id.btnActualizar)
    @Override
    public void actualizarDatos() {
        if (locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER) || locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
            actualizarLista();
        } else {
            activarLocalizacion();
        }
    }

    /*
     * Mostramos un toast en pantalla
     */
    @Override
    public void mostrarToast(String mensaje) {
        Toast.makeText(getActivity().getApplicationContext(), mensaje, Toast.LENGTH_LONG).show();
    }

    /*
     * Obtenemos la localización del usuario por GPS o por RED
     */
    @Override
    public void obtenerNuevaPosicion() {
        locationListener = new MyLocationListener();
        locationManager = (LocationManager) getActivity().getApplicationContext().getSystemService(getActivity().getApplicationContext().LOCATION_SERVICE);

        if (checkLocationPermission(getActivity())) {
            //String provider = locationManager.getBestProvider(criteria, true);
            if (!locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER))
                locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0.0f, locationListener);
            else
                locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0.0f, locationListener);
        }
    }

    /*
     * Accedemos al detalle de la ruta.
     */
    @Override
    public void onItemClick(DatosRuta ruta) {
        ((PantallaInicialActivity) getActivity()).mostrarDetalleRutaServidor(ruta);
    }

    /*
     * Acción que refresca la lista y añade más rutas a la lista
     */
    @Override
    public void onRefresh(SwipyRefreshLayoutDirection direction) {
        refreshSwipe = true;
        new Handler().postDelayed(new Runnable() {
            @Override
            public void run() {
                if(presenter.getIsFiltroAplicados()) {
                    new obtenerRutasServidorConParametros().execute(presenter.getRutasServidor().size());
                }else{
                    new obtenerRutasPorCercaniaUsuario().execute(presenter.getRutasServidor().size());
                }
            }
        },2000);
    }

    /*
     * Obtenemos las rutas cercanas al usuario.
     */
    protected class obtenerRutasPorCercaniaUsuario extends AsyncTask<Integer, Void, List<Ruta>> {

        private List<Ruta> listaResults = new ArrayList<>();

        @Override
        protected void onPreExecute() {
            if(!refreshSwipe) {
                mostrarProgressbar(true);
            }
        }

        @Override
        protected List<Ruta> doInBackground(Integer... nextObjects) {

            if (presenter.comprobarConexion()) {

                tenemosConexion = true;

                if (servicioRuta == null) {
                    RutaApi.Builder builder = new RutaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                            .setRootUrl(ConexionBackend.url);

                    servicioRuta = builder.build();
                }

                try {
                    //Obtenemos las rutas cercanas al usuario en un radio máximo de 1 km.
                    listaResults = servicioRuta.getRutasByLocalizacionInicial(Constants.maxDistanciaPorUsuario, DatosPosicion.latitudeInicial, DatosPosicion.longitudeInicial, nextObjects[0],true).execute().getItems();
                } catch (IOException e) {
                    e.printStackTrace();
                }
                servicioRuta = null;
                tenemosConexion = true;
            } else {
                tenemosConexion = false;
            }

            return listaResults;
        }

        @Override
        protected void onPostExecute(List<Ruta> rutas) {

            servicioRuta = null;

            swipeContainer.setRefreshing(false);

            if (progressbar != null && progressbar.getVisibility() == View.VISIBLE) {
                mostrarProgressbar(false);
            }

            if (rutas != null && rutas.size() > 0) {
                layoutSinDatos.setVisibility(View.GONE);
                layoutConDatos.setVisibility(View.VISIBLE);

                presenter.setListaRutasServidor(rutas);

                setRutas(rutas);
            } else {
                if(!refreshSwipe) {
                    layoutSinDatos.setVisibility(View.VISIBLE);
                    layoutConDatos.setVisibility(View.GONE);

                    if (!locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && !locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                        btnActualizar.setVisibility(View.VISIBLE);
                        textSinResultados.setText(getString(R.string.detalle_ruta_servidor_sinResultados));
                        mostrarToast(getString(R.string.detalle_ruta_servidor_sinLocalizacion));
                    } else {

                        if (!tenemosConexion) {
                            mostrarToast(getString(R.string.sinConexion));
                            btnActualizar.setVisibility(View.VISIBLE);
                        } else {
                            btnActualizar.setVisibility(View.GONE);
                            textSinResultados.setText(getString(R.string.detalle_ruta_servidor_sinResultados));
                        }
                    }
                }
            }
        }
    }


    /*
     * Método asincrono para obtener las rutas compartidas con unos determinados parámetros.
     */
    protected class obtenerRutasServidorConParametros extends AsyncTask<Integer, Void, List<Ruta>> {

        private List<Ruta> listaResults = new ArrayList<>();

        @Override
        protected void onPreExecute() {
            if(!refreshSwipe) {
                mostrarProgressbar(true);
            }
        }

        @Override
        protected List<Ruta> doInBackground(Integer... numObjects) {

            if (presenter.comprobarConexion()) {

                tenemosConexion = true;

                if (servicioRuta == null) {
                    RutaApi.Builder builder = new RutaApi.Builder(AndroidHttp.newCompatibleTransport(), new AndroidJsonFactory(), null)
                            .setRootUrl(ConexionBackend.url);

                    servicioRuta = builder.build();
                }

                try {
                    listaResults = servicioRuta.getRutasByParametros(DatosFiltros.idDifFisica, DatosFiltros.idDifTecnica, DatosFiltros.distancia, DatosPosicion.latitudeInicial, DatosPosicion.longitudeInicial, DatosFiltros.lugar, Constants.maxDistanciaPorUsuario, DatosFiltros.idTipoRuta, numObjects[0]).execute().getItems();
                } catch (IOException e) {
                    e.printStackTrace();
                }

                servicioRuta = null;
                tenemosConexion = true;
            } else {
                tenemosConexion = false;
            }
            return listaResults;
        }

        @Override
        protected void onPostExecute(List<Ruta> rutas) {

            if(!refreshSwipe) {
                if (progressbar != null && progressbar.getVisibility() == View.VISIBLE) {
                    mostrarProgressbar(false);
                }
            }else{
                swipeContainer.setRefreshing(false);
            }

            if (rutas != null && rutas.size() > 0) {
                layoutSinDatos.setVisibility(View.GONE);
                layoutConDatos.setVisibility(View.VISIBLE);

                presenter.setListaRutasServidor(rutas);

                setRutas(rutas);
            } else {
                if(!refreshSwipe) {
                    layoutSinDatos.setVisibility(View.VISIBLE);
                    layoutConDatos.setVisibility(View.GONE);

                    if (!locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER) && !locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                        btnActualizar.setVisibility(View.VISIBLE);
                        textSinResultados.setText(getString(R.string.detalle_ruta_servidor_sinLocalizacion));
                    } else {

                        if (!tenemosConexion) {
                            btnActualizar.setVisibility(View.VISIBLE);
                            textSinResultados.setText(getString(R.string.misRutas_sinConexion));
                            mostrarToast(getString(R.string.detalle_ruta_servidor_sinLocalizacion));
                        } else {
                            btnActualizar.setVisibility(View.GONE);
                            textSinResultados.setText(getString(R.string.detalle_ruta_servidor_sinResultados));
                        }
                    }
                }
            }
        }
    }

    /*
     * Comprobamos si está activda la localización del usuario
     */
    public static boolean checkLocationPermission(Activity activity) {
        if (ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED
                || ActivityCompat.checkSelfPermission(activity, Manifest.permission.ACCESS_COARSE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {

            ActivityCompat.requestPermissions(activity, PERMISSIONS, PERMISSION_ALL);
            return false;
        }
        return true;
    }

    public class MyLocationListener implements LocationListener {

        @Override
        public void onLocationChanged(Location location) {
            DatosPosicion.latitudeInicial = location.getLatitude();
            DatosPosicion.longitudeInicial = location.getLongitude();
        }

        @Override
        public void onStatusChanged(String s, int i, Bundle bundle) {

        }

        @Override
        public void onProviderEnabled(String s) {

        }

        @Override
        public void onProviderDisabled(String s) {

        }
    }
}
